from conan.errors import ConanException
from conan.internal.model.version import Version


def disable_flag(conanfile, flag):
    disable_flags = conanfile.conf.get("tools.gnu:disable_flags", check_type=list)
    if disable_flags is None:
        return False
    valid = ["arch", "arch_link", "libcxx", "build_type", "build_type_link", "threads",
             "cppstd", "cstd"]
    for v in disable_flags:
        if v not in valid:
            raise ConanException(f"tools.gnu:disable_flags value '{v}', must be one of: {valid}")
    return flag in disable_flags


def architecture_flag(conanfile):
    """
    returns flags specific to the target architecture and compiler
    Used by CMakeToolchain and AutotoolsToolchain
    """
    if disable_flag(conanfile, "arch"):
        return ""
    settings = conanfile.settings
    from conan.tools.apple.apple import _to_apple_arch
    compiler = settings.get_safe("compiler")
    arch = settings.get_safe("arch")
    the_os = settings.get_safe("os")
    subsystem = settings.get_safe("os.subsystem")
    subsystem_ios_version = settings.get_safe("os.subsystem.ios_version")
    if not compiler or not arch:
        return ""

    if the_os == "Android":
        return ""

    if compiler == "clang" and the_os == "Windows":
        comp_exes = conanfile.conf.get("tools.build:compiler_executables", check_type=dict,
                                       default={})
        clangcl = "clang-cl" in (comp_exes.get("c") or comp_exes.get("cpp", ""))
        if clangcl:
            return ""  # Do not add arch flags for clang-cl, can happen in cross-build runtime=None
        # LLVM/Clang and VS/Clang must define runtime. msys2 clang won't
        runtime = settings.get_safe("compiler.runtime")  # runtime is Windows only
        if runtime is not None:
            return ""
        # TODO: Maybe Clang-Mingw runtime does, but with C++ is impossible to test
        return {"x86_64": "-m64",
                "x86": "-m32"}.get(arch, "")
    elif compiler in ['gcc', 'apple-clang', 'clang', 'sun-cc']:
        if the_os == 'Macos' and subsystem == 'catalyst':
            # FIXME: This might be conflicting with Autotools --target cli arg
            apple_arch = _to_apple_arch(arch)
            if apple_arch:
                # TODO: Could we define anything like `to_apple_target()`?
                #       Check https://github.com/rust-lang/rust/issues/48862
                return f'--target={apple_arch}-apple-ios{subsystem_ios_version}-macabi'
        elif arch in ['x86_64', 'sparcv9', 's390x']:
            return '-m64'
        elif arch in ['x86', 'sparc']:
            return '-m32'
        elif arch in ['s390']:
            return '-m31'
        elif arch in ['tc131', 'tc16', 'tc161', 'tc162', 'tc18']:
            return '-m{}'.format(arch)
        elif the_os == 'AIX':
            if arch in ['ppc32']:
                return '-maix32'
            elif arch in ['ppc64']:
                return '-maix64'
    elif compiler == "intel-cc":
        # https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-m32-m64-qm32-qm64
        if arch == "x86":
            return "/Qm32" if the_os == "Windows" else "-m32"
        elif arch == "x86_64":
            return "/Qm64" if the_os == "Windows" else "-m64"
    elif compiler == "mcst-lcc":
        return {"e2k-v2": "-march=elbrus-v2",
                "e2k-v3": "-march=elbrus-v3",
                "e2k-v4": "-march=elbrus-v4",
                "e2k-v5": "-march=elbrus-v5",
                "e2k-v6": "-march=elbrus-v6",
                "e2k-v7": "-march=elbrus-v7"}.get(arch, "")
    elif compiler == "emcc":
        if arch == "wasm64":
            return "-sMEMORY64=1"
    return ""


def architecture_link_flag(conanfile):
    """
    returns exclusively linker flags specific to the target architecture and compiler
    """
    if disable_flag(conanfile, "arch_link"):
        return ""
    compiler = conanfile.settings.get_safe("compiler")
    arch = conanfile.settings.get_safe("arch")
    if compiler == "emcc":
        # Emscripten default output is WASM since 1.37.x (long time ago)
        # Deactivate WASM output forcing asm.js output instead
        if arch == "asm.js":
            return "-sWASM=0"
    return ""


def libcxx_flags(conanfile):
    libcxx = conanfile.settings.get_safe("compiler.libcxx")
    if not libcxx:
        return None, None
    if disable_flag(conanfile, "libcxx"):
        return None, None
    compiler = conanfile.settings.get_safe("compiler")
    lib = stdlib11 = None
    if compiler == "apple-clang":
        # In apple-clang 2 only values atm are "libc++" and "libstdc++"
        lib = f'-stdlib={libcxx}'
    elif compiler in ("clang", "intel-cc", "emcc"):
        if libcxx == "libc++":
            lib = "-stdlib=libc++"
        elif libcxx == "libstdc++" or libcxx == "libstdc++11":
            lib = "-stdlib=libstdc++"
        # FIXME, something to do with the other values? Android c++_shared?
    elif compiler == "sun-cc":
        lib = {"libCstd": "-library=Cstd",
               "libstdcxx": "-library=stdcxx4",
               "libstlport": "-library=stlport4",
               "libstdc++": "-library=stdcpp"
               }.get(libcxx)
    elif compiler == "qcc":
        lib = f'-Y _{libcxx}'

    if compiler in ['clang', 'apple-clang', 'gcc', 'emcc']:
        if libcxx == "libstdc++":
            stdlib11 = "_GLIBCXX_USE_CXX11_ABI=0"
        elif libcxx == "libstdc++11" and conanfile.conf.get("tools.gnu:define_libcxx11_abi",
                                                            check_type=bool):
            stdlib11 = "_GLIBCXX_USE_CXX11_ABI=1"
    return lib, stdlib11


def build_type_link_flags(settings):
    """
    returns link flags specific to the build type (Debug, Release, etc.)
    [-debug]
    """
    compiler = settings.get_safe("compiler")
    build_type = settings.get_safe("build_type")
    if not compiler or not build_type:
        return []

    # https://github.com/Kitware/CMake/blob/d7af8a34b67026feaee558433db3a835d6007e06/
    # Modules/Platform/Windows-MSVC.cmake
    if compiler == "msvc":
        if build_type in ("Debug", "RelWithDebInfo"):
            return ["-debug"]

    return []


def build_type_flags(conanfile):
    """
    returns flags specific to the build type (Debug, Release, etc.)
    (-s, -g, /Zi, etc.)
    Used only by AutotoolsToolchain
    """
    if disable_flag(conanfile, "build_type"):
        return []
    settings = conanfile.settings
    compiler = settings.get_safe("compiler")
    build_type = settings.get_safe("build_type")
    vs_toolset = settings.get_safe("compiler.toolset")
    if not compiler or not build_type:
        return []

    comp_exes = conanfile.conf.get("tools.build:compiler_executables", check_type=dict,
                                   default={})
    clangcl = "clang-cl" in (comp_exes.get("c") or comp_exes.get("cpp", ""))

    if compiler == "msvc" or clangcl:
        # https://github.com/Kitware/CMake/blob/d7af8a34b67026feaee558433db3a835d6007e06/
        # Modules/Platform/Windows-MSVC.cmake
        # FIXME: This condition seems legacy, as no more "clang" exists in Conan toolsets
        if vs_toolset and "clang" in vs_toolset:
            flags = {"Debug": ["-gline-tables-only", "-fno-inline", "-O0"],
                     "Release": ["-O2"],
                     "RelWithDebInfo": ["-gline-tables-only", "-O2", "-fno-inline"],
                     "MinSizeRel": []
                     }.get(build_type, ["-O2", "-Ob2"])
        else:
            flags = {"Debug": ["-Zi", "-Ob0", "-Od"],
                     "Release": ["-O2", "-Ob2"],
                     "RelWithDebInfo": ["-Zi", "-O2", "-Ob1"],
                     "MinSizeRel": ["-O1", "-Ob1"],
                     }.get(build_type, [])
        return flags
    else:
        # https://github.com/Kitware/CMake/blob/f3bbb37b253a1f4a26809d6f132b3996aa2e16fc/
        # Modules/Compiler/GNU.cmake
        # clang include the gnu (overriding some things, but not build type) and apple clang
        # overrides clang but it doesn't touch clang either
        if compiler in ["clang", "gcc", "apple-clang", "qcc", "mcst-lcc"]:
            flags = {"Debug": ["-g"],
                     "Release": ["-O3"],
                     "RelWithDebInfo": ["-O2", "-g"],
                     "MinSizeRel": ["-Os"],
                     }.get(build_type, [])
            return flags
        elif compiler == "sun-cc":
            # https://github.com/Kitware/CMake/blob/f3bbb37b253a1f4a26809d6f132b3996aa2e16fc/
            # Modules/Compiler/SunPro-CXX.cmake
            flags = {"Debug": ["-g"],
                     "Release": ["-xO3"],
                     "RelWithDebInfo": ["-xO2", "-g"],
                     "MinSizeRel": ["-xO2", "-xspace"],
                     }.get(build_type, [])
            return flags
    return []


def threads_flags(conanfile):
    """
    returns flags specific to the threading model used by the compiler
    """
    if disable_flag(conanfile, "threads"):
        return []
    compiler = conanfile.settings.get_safe("compiler")
    threads = conanfile.settings.get_safe("compiler.threads")
    if compiler == "emcc":
        if threads == "posix":
            return ["-pthread"]
        elif threads == "wasm_workers":
            return ["-sWASM_WORKERS=1"]
    return []


def llvm_clang_front(conanfile):
    # Only Windows clang with MSVC backend (LLVM/Clang, not MSYS2 clang)
    if (conanfile.settings.get_safe("os") != "Windows" or
            conanfile.settings.get_safe("compiler") != "clang" or
            not conanfile.settings.get_safe("compiler.runtime")):
        return
    compilers = conanfile.conf.get("tools.build:compiler_executables", default={})
    if "clang-cl" in compilers.get("c", "") or "clang-cl" in compilers.get("cpp", ""):
        return "clang-cl"  # The MSVC-compatible front
    return "clang"  # The GNU-compatible front


def cppstd_flag(conanfile) -> str:
    """
    Returns flags specific to the C++ standard based on the ``conanfile.settings.compiler``,
    ``conanfile.settings.compiler.version`` and ``conanfile.settings.compiler.cppstd``.

    It also considers when using GNU extension in ``settings.compiler.cppstd``, reflecting it in the
    compiler flag. Currently, it supports GCC, Clang, AppleClang, MSVC, Intel, MCST-LCC.

    In case there is no ``settings.compiler`` or ``settings.cppstd`` in the profile, the result will
    be an **empty string**.

    :param conanfile: The current recipe object. Always use ``self``.
    :return: ``str`` with the standard C++ flag used by the compiler. e.g. "-std=c++11", "/std:c++latest"
    """
    compiler = conanfile.settings.get_safe("compiler")
    compiler_version = conanfile.settings.get_safe("compiler.version")
    cppstd = conanfile.settings.get_safe("compiler.cppstd")

    if not compiler or not compiler_version or not cppstd:
        return ""

    if disable_flag(conanfile, "cppstd"):
        return ""

    func = {"gcc": _cppstd_gcc,
            "clang": _cppstd_clang,
            "apple-clang": _cppstd_apple_clang,
            "msvc": _cppstd_msvc,
            "intel-cc": _cppstd_intel_cc,
            "mcst-lcc": _cppstd_mcst_lcc}.get(compiler)
    flag = None
    if func:
        flag = func(Version(compiler_version), str(cppstd))
    if flag and llvm_clang_front(conanfile) == "clang-cl":
        flag = flag.replace("=", ":")
    return flag


def cppstd_msvc_flag(visual_version, cppstd):
    # https://docs.microsoft.com/en-us/cpp/build/reference/std-specify-language-standard-version
    if cppstd == "23":
        if visual_version >= "193":
            return "c++latest"
    elif cppstd == "20":
        if visual_version >= "192":
            return "c++20"
        elif visual_version >= "191":
            return "c++latest"
    elif cppstd == "17":
        if visual_version >= "191":
            return "c++17"
        elif visual_version >= "190":
            return "c++latest"
    elif cppstd == "14":
        if visual_version >= "190":
            return "c++14"

    return None


def _cppstd_msvc(visual_version, cppstd):
    flag = cppstd_msvc_flag(visual_version, cppstd)
    return f'/std:{flag}' if flag else None


def _cppstd_apple_clang(clang_version, cppstd):
    """
    Inspired in:
    https://github.com/Kitware/CMake/blob/master/Modules/Compiler/AppleClang-CXX.cmake
    """

    v98 = vgnu98 = v11 = vgnu11 = v14 = vgnu14 = v17 = vgnu17 = v20 = vgnu20 = v23 = vgnu23 = v26 = vgnu26 = None

    if clang_version >= "4.0":
        v98 = "c++98"
        vgnu98 = "gnu++98"
        v11 = "c++11"
        vgnu11 = "gnu++11"

    if clang_version >= "6.1":
        v14 = "c++14"
        vgnu14 = "gnu++14"
    elif clang_version >= "5.1":
        v14 = "c++1y"
        vgnu14 = "gnu++1y"

    # Not confirmed that it didn't work before 9.1 but 1z is still valid, so we are ok
    # Note: cmake allows c++17 since version 10.0
    if clang_version >= "9.1":
        v17 = "c++17"
        vgnu17 = "gnu++17"
    elif clang_version >= "6.1":
        v17 = "c++1z"
        vgnu17 = "gnu++1z"

    if clang_version >= "13.0":
        v20 = "c++20"
        vgnu20 = "gnu++20"
    elif clang_version >= "10.0":
        v20 = "c++2a"
        vgnu20 = "gnu++2a"

    if clang_version >= "16.0":
        v23 = "c++23"
        vgnu23 = "gnu++23"

        v26 = "c++26"
        vgnu26 = "gnu++26"
    elif clang_version >= "13.0":
        v23 = "c++2b"
        vgnu23 = "gnu++2b"

    flag = {"98": v98, "gnu98": vgnu98,
            "11": v11, "gnu11": vgnu11,
            "14": v14, "gnu14": vgnu14,
            "17": v17, "gnu17": vgnu17,
            "20": v20, "gnu20": vgnu20,
            "23": v23, "gnu23": vgnu23,
            "26": v26, "gnu26": vgnu26}.get(cppstd)

    return f'-std={flag}' if flag else None


def _cppstd_clang(clang_version, cppstd):
    """
    Inspired in:
    https://github.com/Kitware/CMake/blob/
    1fe2dc5ef2a1f262b125a2ba6a85f624ce150dd2/Modules/Compiler/Clang-CXX.cmake

    https://clang.llvm.org/cxx_status.html
    """
    v98 = vgnu98 = v11 = vgnu11 = v14 = vgnu14 = v17 = vgnu17 = v20 = vgnu20 = v23 = vgnu23 = v26 = vgnu26 = None

    if clang_version >= "2.1":
        v98 = "c++98"
        vgnu98 = "gnu++98"

    if clang_version >= "3.1":
        v11 = "c++11"
        vgnu11 = "gnu++11"
    elif clang_version >= "2.1":
        v11 = "c++0x"
        vgnu11 = "gnu++0x"

    if clang_version >= "3.5":
        v14 = "c++14"
        vgnu14 = "gnu++14"
    elif clang_version >= "3.4":
        v14 = "c++1y"
        vgnu14 = "gnu++1y"

    if clang_version >= "5":
        v17 = "c++17"
        vgnu17 = "gnu++17"
    elif clang_version >= "3.5":
        v17 = "c++1z"
        vgnu17 = "gnu++1z"

    if clang_version >= "6":
        v20 = "c++2a"
        vgnu20 = "gnu++2a"

    if clang_version >= "12":
        v20 = "c++20"
        vgnu20 = "gnu++20"

        v23 = "c++2b"
        vgnu23 = "gnu++2b"

    if clang_version >= "17":
        v23 = "c++23"
        vgnu23 = "gnu++23"

        v26 = "c++26"
        vgnu26 = "gnu++26"

    flag = {"98": v98, "gnu98": vgnu98,
            "11": v11, "gnu11": vgnu11,
            "14": v14, "gnu14": vgnu14,
            "17": v17, "gnu17": vgnu17,
            "20": v20, "gnu20": vgnu20,
            "23": v23, "gnu23": vgnu23,
            "26": v26, "gnu26": vgnu26}.get(cppstd)
    return f'-std={flag}' if flag else None


def _cppstd_gcc(gcc_version, cppstd):
    """https://github.com/Kitware/CMake/blob/master/Modules/Compiler/GNU-CXX.cmake"""
    # https://gcc.gnu.org/projects/cxx-status.html
    v98 = vgnu98 = v11 = vgnu11 = v14 = vgnu14 = v17 = vgnu17 = v20 = vgnu20 = v23 = vgnu23 = v26 = vgnu26 = None

    if gcc_version >= "3.4":
        v98 = "c++98"
        vgnu98 = "gnu++98"

    if gcc_version >= "4.7":
        v11 = "c++11"
        vgnu11 = "gnu++11"
    elif gcc_version >= "4.3":
        v11 = "c++0x"
        vgnu11 = "gnu++0x"

    if gcc_version >= "4.9":
        v14 = "c++14"
        vgnu14 = "gnu++14"
    elif gcc_version >= "4.8":
        v14 = "c++1y"
        vgnu14 = "gnu++1y"

    if gcc_version >= "5":
        v17 = "c++1z"
        vgnu17 = "gnu++1z"

    if gcc_version >= "5.2":  # Not sure if even in 5.1 gnu17 is valid, but gnu1z is
        v17 = "c++17"
        vgnu17 = "gnu++17"

    if gcc_version >= "8":
        v20 = "c++2a"
        vgnu20 = "gnu++2a"

    if gcc_version >= "10":
        v20 = "c++20"
        vgnu20 = "gnu++20"

    if gcc_version >= "11":
        v23 = "c++23"
        vgnu23 = "gnu++23"

    if gcc_version >= "14":
        v26 = "c++26"
        vgnu26 = "gnu++26"

    flag = {"98": v98, "gnu98": vgnu98,
            "11": v11, "gnu11": vgnu11,
            "14": v14, "gnu14": vgnu14,
            "17": v17, "gnu17": vgnu17,
            "20": v20, "gnu20": vgnu20,
            "23": v23, "gnu23": vgnu23,
            "26": v26, "gnu26": vgnu26}.get(cppstd)
    return f'-std={flag}' if flag else None


def _cppstd_mcst_lcc(mcst_lcc_version, cppstd):
    v11 = vgnu11 = v14 = vgnu14 = v17 = vgnu17 = v20 = vgnu20 = None

    if mcst_lcc_version >= "1.21":
        v11 = "c++11"
        vgnu11 = "gnu++11"
        v14 = "c++14"
        vgnu14 = "gnu++14"

    if mcst_lcc_version >= "1.24":
        v17 = "c++17"
        vgnu17 = "gnu++17"

    if mcst_lcc_version >= "1.25":
        v20 = "c++2a"
        vgnu20 = "gnu++2a"

    # FIXME: What is this "03"?? that is not a valid cppstd in the settings.yml
    flag = {"98": "c++98", "gnu98": "gnu++98",
            "03": "c++03", "gnu03": "gnu++03",
            "11": v11, "gnu11": vgnu11,
            "14": v14, "gnu14": vgnu14,
            "17": v17, "gnu17": vgnu17,
            "20": v20, "gnu20": vgnu20}.get(cppstd)
    return f'-std={flag}' if flag else None


def _cppstd_intel_cc(_, cppstd):
    """
    Inspired in:
    https://software.intel.com/content/www/us/en/develop/documentation/
    oneapi-dpcpp-cpp-compiler-dev-guide-and-reference/top/compiler-reference/
    compiler-options/compiler-option-details/language-options/std-qstd.html
    """
    # Note: for now, we don't care about compiler version
    v98 = "c++98"
    vgnu98 = "gnu++98"
    v03 = "c++03"
    vgnu03 = "gnu++03"
    v11 = "c++11"
    vgnu11 = "gnu++11"
    v14 = "c++14"
    vgnu14 = "gnu++14"
    v17 = "c++17"
    vgnu17 = "gnu++17"
    v20 = "c++20"
    vgnu20 = "gnu++20"
    v23 = "c++2b"
    vgnu23 = "gnu++2b"

    flag = {"98": v98, "gnu98": vgnu98,
            "03": v03, "gnu03": vgnu03,
            "11": v11, "gnu11": vgnu11,
            "14": v14, "gnu14": vgnu14,
            "17": v17, "gnu17": vgnu17,
            "20": v20, "gnu20": vgnu20,
            "23": v23, "gnu23": vgnu23}.get(cppstd)
    return f'-std={flag}' if flag else None


def cstd_flag(conanfile) -> str:
    """
    Returns flags specific to the C+standard based on the ``conanfile.settings.compiler``,
    ``conanfile.settings.compiler.version`` and ``conanfile.settings.compiler.cstd``.

    It also considers when using GNU extension in ``settings.compiler.cstd``, reflecting it in the
    compiler flag. Currently, it supports GCC, Clang, AppleClang, MSVC, Intel, MCST-LCC.

    In case there is no ``settings.compiler`` or ``settings.cstd`` in the profile, the result will
    be an **empty string**.

    :param conanfile: The current recipe object. Always use ``self``.
    :return: ``str`` with the standard C flag used by the compiler.
    """
    compiler = conanfile.settings.get_safe("compiler")
    compiler_version = conanfile.settings.get_safe("compiler.version")
    cstd = conanfile.settings.get_safe("compiler.cstd")

    if not compiler or not compiler_version or not cstd:
        return ""

    if disable_flag(conanfile, "cstd"):
        return ""

    func = {"gcc": _cstd_gcc,
            "clang": _cstd_clang,
            "apple-clang": _cstd_apple_clang,
            "msvc": _cstd_msvc}.get(compiler)
    flag = None
    if func:
        flag = func(Version(compiler_version), str(cstd))
    return flag


def _cstd_gcc(gcc_version, cstd):
    # TODO: Verify flags per version
    flag = {"99": "c99",
            "11": "c11",
            "17": "c17",
            "23": "c23"}.get(cstd, cstd)
    return f'-std={flag}' if flag else None


def _cstd_clang(gcc_version, cstd):
    # TODO: Verify flags per version
    flag = {"99": "c99",
            "11": "c11",
            "17": "c17",
            "23": "c23"}.get(cstd, cstd)
    return f'-std={flag}' if flag else None


def _cstd_apple_clang(gcc_version, cstd):
    # TODO: Verify flags per version
    flag = {"99": "c99",
            "11": "c11",
            "17": "c17",
            "23": "c23"}.get(cstd, cstd)
    return f'-std={flag}' if flag else None


def cstd_msvc_flag(visual_version, cstd):
    if cstd == "17":
        if visual_version >= "192":
            return "c17"
    elif cstd == "11":
        if visual_version >= "192":
            return "c11"
    return None


def _cstd_msvc(visual_version, cstd):
    flag = cstd_msvc_flag(visual_version, cstd)
    return f'/std:{flag}' if flag else None



from conan.test.utils.tools import TestClient


class TestLoadRequirementsTextFileTest:

    def test_load_reqs_from_text_file(self):
        client = TestClient()
        conanfile = """from conan import ConanFile
def reqs():
    try:
        content = open("reqs.txt", "r").read()
        lines = [line for line in content.splitlines() if line]
        return tuple(lines)
    except:
        return None

class Test(ConanFile):
    exports = "reqs.txt"
    requires = reqs()
"""
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=hello0 --version=0.1 --user=user --channel=channel")

        for i in (0, 1, 2):
            reqs = "hello%s/0.1@user/channel" % i
            client.save({"conanfile.py": conanfile,
                         "reqs.txt": reqs})
            client.run("create . --name=hello%s --version=0.1 --user=user --channel=channel" % (i + 1))

        client.run("install --requires=hello3/0.1@user/channel")
        client.assert_listed_require({"hello0/0.1@user/channel": "Cache",
                                      "hello1/0.1@user/channel": "Cache",
                                      "hello2/0.1@user/channel": "Cache",
                                      "hello3/0.1@user/channel": "Cache"})



import json
import os

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient, TestServer
from conan.internal.util.files import save


@pytest.fixture()
def client():
    test_server = TestServer()
    c = TestClient(servers={"default": test_server})
    c.save({"conanfile.py": GenConanfile("pkg", "0.1")})
    c.run("create .")
    c.run("upload * -r=default -c", assert_error=True)
    return c


def test_remote_file_credentials(client):
    c = client
    content = {"credentials": [{"remote": "default", "user": "admin", "password": "password"}]}
    save(os.path.join(c.cache_folder, "credentials.json"), json.dumps(content))
    c.run("upload * -r=default -c")
    # it works without problems!
    assert "Uploading recipe" in c.out


def test_remote_file_credentials_remote_login(client):
    c = client
    content = {"credentials": [{"remote": "default", "user": "admin", "password": "password"}]}
    save(os.path.join(c.cache_folder, "credentials.json"), json.dumps(content))
    c.run("remote login default")
    assert "Changed user of remote 'default' from 'None' (anonymous) " \
           "to 'admin' (authenticated)" in c.out


def test_remote_file_credentials_error(client):
    c = client
    content = {"credentials": [{"remote": "default", "user": "admin", "password": "wrong"}]}
    save(os.path.join(c.cache_folder, "credentials.json"), json.dumps(content))
    c.run("upload * -r=default -c", assert_error=True)
    assert "ERROR: Wrong user or password" in c.out


def test_remote_file_credentials_bad_file(client):
    c = client
    save(os.path.join(c.cache_folder, "credentials.json"), "")
    c.run("upload * -r=default -c", assert_error=True)
    assert "ERROR: Error loading 'credentials.json'" in c.out
    content = {"credentials": [{"remote": "default"}]}
    save(os.path.join(c.cache_folder, "credentials.json"), json.dumps(content))
    c.run("upload * -r=default -c", assert_error=True)
    assert "ERROR: Error loading 'credentials.json'" in c.out



import os
import tempfile

from conan.api.output import ConanOutput, LEVEL_STATUS, Color, LEVEL_ERROR, LEVEL_QUIET
from conan.cli.args import common_graph_args, validate_common_graph_args
from conan.cli.command import conan_command
from conan.cli.commands.install import _run_install_command
from conan.errors import ConanException


@conan_command(group="Consumer")
def run(conan_api, parser, *args):
    """
    (Experimental) Run a command given a set of requirements from a recipe or from command line.
    """
    common_graph_args(parser)
    parser.add_argument("command", help="Command to run")
    parser.add_argument("--context", help="Context to use, by default both contexts are activated "
                                          "if not specified",
                        choices=["host", "build"], default=None)
    parser.add_argument("--build-require", action='store_true', default=False,
                        help='Whether the provided path is a build-require')
    args = parser.parse_args(*args)
    validate_common_graph_args(args)
    cwd = os.getcwd()

    ConanOutput().info("Installing and building dependencies, this might take a while...",
                       fg=Color.BRIGHT_MAGENTA)
    previous_log_level = ConanOutput.get_output_level()
    if previous_log_level == LEVEL_STATUS:
        ConanOutput.set_output_level(LEVEL_QUIET)

    with tempfile.TemporaryDirectory("conanrun") as tmpdir:
        # Default values for install
        setattr(args, "output_folder", tmpdir)
        setattr(args, "generator", [])
        try:
            deps_graph, lockfile, _ = _run_install_command(conan_api, args, cwd,
                                                           return_install_error=False)
        except ConanException as e:
            ConanOutput.set_output_level(previous_log_level)
            ConanOutput().error("Error installing the dependencies. To debug this, you can either:\n"
                                " - Re-run the command with increased verbosity (-v, -vv)\n"
                                " - Run 'conan install' first to ensure dependencies are installed, "
                                "or to see errors during installation\n")
            raise e

        context_env_map = {
            "build": "conanbuild",
            "host": "conanrun",
        }
        envfiles = list(context_env_map.values()) if args.context is None \
            else [context_env_map.get(args.context)]
        ConanOutput.set_output_level(LEVEL_ERROR)
        deps_graph.root.conanfile.run(args.command, cwd=cwd, env=envfiles)



import os

from unittest import mock
from unittest.mock import Mock, MagicMock

from conan import __version__
from conan.internal.rest.conan_requester import ConanRequester
from conan.internal.model.conf import ConfDefinition
from conan.test.utils.tools import temp_folder
from conan.internal.util.files import save


class MockRequesterGet(Mock):
    verify = None

    def get(self, _, **kwargs):
        self.verify = kwargs.get('verify', None)


class TestConanRequesterCacertPath:
    def test_default_no_verify(self):
        mocked_requester = MockRequesterGet()
        with mock.patch("conan.internal.rest.conan_requester.requests", mocked_requester):
            requester = ConanRequester(ConfDefinition())
            requester.get(url="aaa", verify=False)
            assert requester._http_requester.verify is False

    def test_default_verify(self):
        mocked_requester = MockRequesterGet()
        with mock.patch("conan.internal.rest.conan_requester.requests", mocked_requester):
            requester = ConanRequester(ConfDefinition())
            requester.get(url="aaa", verify=True)
            assert requester._http_requester.verify is True

    def test_cache_config(self):
        file_path = os.path.join(temp_folder(), "whatever_cacert")
        save(file_path, "")
        config = ConfDefinition()
        config.update("core.net.http:cacert_path", file_path)
        mocked_requester = MockRequesterGet()
        with mock.patch("conan.internal.rest.conan_requester.requests", mocked_requester):
            requester = ConanRequester(config)
            requester.get(url="bbbb", verify=True)
        assert requester._http_requester.verify == file_path


class TestConanRequesterHeaders:
    def test_user_agent(self):
        mock_http_requester = MagicMock()
        with mock.patch("conan.internal.rest.conan_requester.requests", mock_http_requester):
            requester = ConanRequester(ConfDefinition())
            requester.get(url="aaa")
            headers = requester._http_requester.get.call_args[1]["headers"]
            assert "Conan/%s" % __version__ in headers["User-Agent"]

            requester.get(url="aaa", headers={"User-Agent": "MyUserAgent"})
            headers = requester._http_requester.get.call_args[1]["headers"]
            assert "MyUserAgent" == headers["User-Agent"]



import textwrap

import pytest

from conan.test.utils.tools import GenConanfile, TestClient, NO_SETTINGS_PACKAGE_ID

PKG_ID_NO_CONF = "ebec3dc6d7f6b907b3ada0c3d3cdc83613a2b715"
PKG_ID_1 = "89d32f25195a77f4ae2e77414b870781853bdbc1"
PKG_ID_2 = "7f9ed92704709f56ecc7b133322479caf3ffd7ad"
PKG_ID_3 = "a9f917f3bad3b48b5bceae70214764274ccd6337"
PKG_ID_USER_1 = "571b5dae13b37a78c1993842739bd475879092ea"
PKG_ID_USER_2 = "54a394d26f9c35add86f20ac02cacc3c7e18f02c"
PKG_ID_USER_3 = "54a394d26f9c35add86f20ac02cacc3c7e18f02c"


@pytest.mark.parametrize("package_id_confs, package_id", [
    ('[]', PKG_ID_NO_CONF),
    ('["user.fake:no_existing_conf"]', PKG_ID_NO_CONF),
    ('["tools.build:cxxflags", "tools.build:cflags"]', PKG_ID_1),
    ('["tools.build:defines"]', PKG_ID_2),
    ('["tools.build:cxxflags", "tools.build:sharedlinkflags"]', PKG_ID_3),
    ('["user.foo:value"]', PKG_ID_USER_1),
    ('["user.foo:value", "user.bar:value"]', PKG_ID_USER_2),
    ('["user.*"]', PKG_ID_USER_3),
])
def test_package_id_including_confs(package_id_confs, package_id):
    client = TestClient()
    profile = textwrap.dedent(f"""
    include(default)
    [conf]
    tools.info.package_id:confs={package_id_confs}
    tools.build:cxxflags=["--flag1", "--flag2"]
    tools.build:cflags+=["--flag3", "--flag4"]
    tools.build:sharedlinkflags=+["--flag5", "--flag6"]
    tools.build:exelinkflags=["--flag7", "--flag8"]
    tools.build:defines=["D1", "D2"]

    user.foo:value=1
    user.bar:value=2
    """)
    client.save({"conanfile.py": GenConanfile("pkg", "0.1").with_settings("os"),
                 "profile": profile})
    client.run('create . -s os=Windows -pr profile')
    client.assert_listed_binary({"pkg/0.1": (package_id, "Build")})


PKG_ID_4 = "9b334fc314f2f2ce26e5280901eabcdd7b3f55a6"
PKG_ID_5 = "5510413d2e6186662cb473fb16ce0a18a3f9e98f"


@pytest.mark.parametrize("cxx_flags, package_id", [
    ('[]', PKG_ID_NO_CONF),
    ('["--flag1", "--flag2"]', PKG_ID_4),
    ('["--flag3", "--flag4"]', PKG_ID_5),
])
def test_same_package_id_configurations_but_changing_values(cxx_flags, package_id):
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os"
        """)
    profile = textwrap.dedent(f"""
    include(default)
    [conf]
    tools.info.package_id:confs=["tools.build:cxxflags"]
    tools.build:cxxflags={cxx_flags}
    tools.build:cflags+=["--flag3", "--flag4"]
    tools.build:sharedlinkflags=+["--flag5", "--flag6"]
    tools.build:exelinkflags=["--flag7", "--flag8"]
    tools.build:defines=["D1", "D2"]
    """)
    client.save({"conanfile.py": conanfile, "profile": profile})
    client.run('create . --name=pkg --version=0.1 -s os=Windows -pr profile')
    client.assert_listed_binary({"pkg/0.1": (package_id, "Build")})


def test_package_id_confs_header_only():
    """
    The tools.info.package_id:confs cannot affect header-only libraries
    and any other library that does ``self.info.clear()`` in ``package_id()`` method
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            package_type = "header-library"
            implements = ["auto_header_only"]
        """)
    profile = textwrap.dedent(f"""
        include(default)
        [conf]
        tools.info.package_id:confs=["tools.build:cxxflags"]
        """)
    client.save({"conanfile.py": conanfile, "profile": profile})
    client.run('create . --name=pkg --version=0.1 -pr profile -c tools.build:cxxflags=["--flag1"]')
    client.assert_listed_binary({"pkg/0.1": (NO_SETTINGS_PACKAGE_ID, "Build")})
    client.run("list *:*")
    assert "tools.build:cxxflags" not in client.out
    client.run('create . --name=pkg --version=0.1 -pr profile -c tools.build:cxxflags=["--flag2"]')
    client.assert_listed_binary({"pkg/0.1": (NO_SETTINGS_PACKAGE_ID, "Build")})
    client.run("list *:*")
    assert "tools.build:cxxflags" not in client.out


def test_conf_pkg_id_user_pattern_not_defined():
    tc = TestClient(light=True)
    tc.save({"lib/conanfile.py": GenConanfile("lib", "1.0")})
    tc.save_home({"global.conf": "tools.info.package_id:confs=['user.*']"})

    # This used to break the build because `user.*` conf was not valid
    tc.run("create lib")
    assert "lib/1.0: Package 'da39a3ee5e6b4b0d3255bfef95601890afd80709' created" in tc.out


@pytest.mark.parametrize("conf,pkgconf", [
    ("user.foo:value=1\nuser.bar:value=2", "['user.foo:value', 'user.bar:value']"),
    ("user.foo:value=1\nuser.bar:value=2", "['user.bar:value', 'user.foo:value']"),
    ("user.bar:value=2\nuser.foo:value=1", "['user.foo:value', 'user.bar:value']"),
    ("user.bar:value=2\nuser.foo:value=1", "['user.bar:value', 'user.foo:value']"),
])
def test_package_id_order(conf, pkgconf):
    """Ensure the order of the definitions in the conf file does not affect the package_id"""
    tc = TestClient(light=True)
    tc.save({"profile": f"[conf]\ntools.info.package_id:confs={pkgconf}\n" +
                        conf,
             "conanfile.py": GenConanfile("pkg", "1.0")})
    tc.run("create . -pr=profile")
    # They should all have the same pkg id - note that this did not happen before 2.0.17
    tc.assert_listed_binary({"pkg/1.0": ("43227c40b8725e89d30a9f97c0652629933a3685", "Build")})



import os
from textwrap import dedent

import pytest

from conan.internal.loader import ConanFileLoader
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save, load

base_conanfile = '''
from conan import ConanFile
from conan.tools.files import patch, replace_in_file
import os

class ConanFileToolsTest(ConanFile):
    name = "test"
    version = "1.9.10"
'''


class TestToolsFilesPatch:

    @pytest.mark.parametrize("strip", [0, 1])
    def test_patch_from_file(self, strip):
        if strip:
            file_content = base_conanfile + '''
    def build(self):
        patch(self, patch_file="file.patch", strip=%s)
''' % strip
            patch_content = '''--- %s/text.txt\t2016-01-25 17:57:11.452848309 +0100
+++ %s/text_new.txt\t2016-01-25 17:57:28.839869950 +0100
@@ -1 +1 @@
-ONE TWO THREE
+ONE TWO FOUR''' % ("old_path", "new_path")
        else:
            file_content = base_conanfile + '''
    def build(self):
        patch(self, patch_file="file.patch")
'''
            patch_content = '''--- text.txt\t2016-01-25 17:57:11.452848309 +0100
+++ text_new.txt\t2016-01-25 17:57:28.839869950 +0100
@@ -1 +1 @@
-ONE TWO THREE
+ONE TWO FOUR'''

        tmp_dir, file_path, text_file = self._save_files(file_content)
        patch_file = os.path.join(tmp_dir, "file.patch")
        save(patch_file, patch_content)
        self._build_and_check(tmp_dir, file_path, text_file, "ONE TWO FOUR")

    def test_patch_from_str(self):
        file_content = base_conanfile + '''
    def build(self):
        patch_content = \'''--- text.txt\t2016-01-25 17:57:11.452848309 +0100
+++ text_new.txt\t2016-01-25 17:57:28.839869950 +0100
@@ -1 +1 @@
-ONE TWO THREE
+ONE TWO DOH!\'''
        patch(self, patch_string=patch_content)

'''
        tmp_dir, file_path, text_file = self._save_files(file_content)
        self._build_and_check(tmp_dir, file_path, text_file, "ONE TWO DOH!")

    def test_patch_strip_new(self):
        conanfile = dedent("""
            from conan import ConanFile
            from conan.tools.files import patch
            class PatchConan(ConanFile):
                def source(self):
                    patch(self, self.source_folder, "example.patch", strip=1)""")
        patch = dedent("""
            --- /dev/null
            +++ b/src/newfile
            @@ -0,0 +0,1 @@
            +New file!""")

        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "example.patch": patch})
        client.run("source .")
        assert client.load("newfile") == "New file!"

    def test_patch_strip_delete(self):
        conanfile = dedent("""
            from conan import ConanFile
            from conan.tools.files import patch
            class PatchConan(ConanFile):
                def source(self):
                    patch(self, self.source_folder, "example.patch", strip=1)""")
        patch = dedent(r"""
            --- a\src\oldfile
            +++ b/dev/null
            @@ -0,1 +0,0 @@
            -legacy code""")
        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "example.patch": patch,
                     "oldfile": "legacy code"})
        path = os.path.join(client.current_folder, "oldfile")
        assert os.path.exists(path)
        client.run("source .")
        assert not os.path.exists(path)

    def test_patch_strip_delete_no_folder(self):
        conanfile = dedent("""
            from conan import ConanFile
            from conan.tools.files import patch
            class PatchConan(ConanFile):
                def source(self):
                    patch(self, self.source_folder, "example.patch", strip=1)""")
        patch = dedent("""
            --- a/oldfile
            +++ b/dev/null
            @@ -0,1 +0,0 @@
            -legacy code""")
        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "example.patch": patch,
                     "oldfile": "legacy code"})
        path = os.path.join(client.current_folder, "oldfile")
        assert os.path.exists(path)
        client.run("source .")
        assert not os.path.exists(path)

    def test_patch_new_delete(self):
        conanfile = base_conanfile + '''
    def build(self):
        from conan.tools.files import load, save
        save(self, "oldfile", "legacy code")
        assert(os.path.exists("oldfile"))
        patch_content = """--- /dev/null
+++ b/newfile
@@ -0,0 +0,3 @@
+New file!
+New file!
+New file!
--- a/oldfile
+++ b/dev/null
@@ -0,1 +0,0 @@
-legacy code
"""
        patch(self, patch_string=patch_content)
        self.output.info("NEW FILE=%s" % load(self, "newfile"))
        self.output.info("OLD FILE=%s" % os.path.exists("oldfile"))
'''
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . --user=user --channel=testing")
        assert "test/1.9.10@user/testing: NEW FILE=New file!\nNew file!\nNew file!\n" in client.out
        assert "test/1.9.10@user/testing: OLD FILE=False" in client.out

    def test_patch_new_strip(self):
        conanfile = base_conanfile + '''
    def build(self):
        from conan.tools.files import load, save
        patch_content = """--- /dev/null
+++ b/newfile
@@ -0,0 +0,3 @@
+New file!
+New file!
+New file!
"""
        patch(self, patch_string=patch_content, strip=1)
        self.output.info("NEW FILE=%s" % load(self, "newfile"))
'''
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . --user=user --channel=testing")
        assert "test/1.9.10@user/testing: NEW FILE=New file!\nNew file!\nNew file!\n" in client.out

    def test_error_patch(self):
        file_content = base_conanfile + '''
    def build(self):
        patch_content = "some corrupted patch"
        patch(self, patch_string=patch_content, output=self.output)

'''
        client = TestClient()
        client.save({"conanfile.py": file_content})
        client.run("install .")
        client.run("build .", assert_error=True)
        assert "patch_ng: error: no patch data found!" in client.out
        assert "ERROR: conanfile.py (test/1.9.10): Error in build() method, line 12" in client.out
        assert "Failed to parse patch: string" in client.out

    def test_add_new_file(self):
        """ Validate issue #5320
        """

        conanfile = dedent("""
            from conan import ConanFile
            from conan.tools.files import patch
            import os

            class ConanFileToolsTest(ConanFile):
                name = "foobar"
                version = "0.1.0"
                exports_sources = "*"

                def build(self):
                    patch(self, patch_file="add_files.patch")
                    assert os.path.isfile("foo.txt")
                    assert os.path.isfile("bar.txt")
        """)
        bar = "no creo en brujas"
        patch = dedent("""
            From c66347c66991b6e617d107b505c18b3115624b8a Mon Sep 17 00:00:00 2001
            From: Uilian Ries <uilianries@gmail.com>
            Date: Wed, 16 Oct 2019 14:31:34 -0300
            Subject: [PATCH] add foo

            ---
             bar.txt | 3 ++-
             foo.txt | 3 +++
             2 files changed, 5 insertions(+), 1 deletion(-)
             create mode 100644 foo.txt

            diff --git a/bar.txt b/bar.txt
            index 0f4ff3a..0bd3158 100644
            --- a/bar.txt
            +++ b/bar.txt
            @@ -1 +1,2 @@
            -no creo en brujas
            +Yo no creo en brujas, pero que las hay, las hay
            +
            diff --git a/foo.txt b/foo.txt
            new file mode 100644
            index 0000000..91e8c0d
            --- /dev/null
            +++ b/foo.txt
            @@ -0,0 +1,3 @@
            +For us, there is no spring.
            +Just the wind that smells fresh before the storm.
            +
            --
            2.23.0


        """)

        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "add_files.patch": patch,
                     "bar.txt": bar})
        client.run("install .")
        client.run("build .")
        bar_content = client.load("bar.txt")
        assert dedent("""Yo no creo en brujas, pero que las hay, las hay
                             """) in bar_content
        foo_content = client.load("foo.txt")
        assert dedent("""For us, there is no spring.
Just the wind that smells fresh before the storm.""") in foo_content
        assert "Calling build()" in client.out
        assert "Warning" not in client.out

    def _save_files(self, file_content):
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "conanfile.py")
        text_file = os.path.join(tmp_dir, "text.txt")
        save(file_path, file_content)
        save(text_file, "ONE TWO THREE")
        return tmp_dir, file_path, text_file

    def _build_and_check(self, tmp_dir, file_path, text_file, msg):
        loader = ConanFileLoader(None)
        ret = loader.load_consumer(file_path)
        curdir = os.path.abspath(os.curdir)
        ret.folders.set_base_source(os.path.dirname(file_path))
        ret.folders.set_base_export_sources(os.path.dirname(file_path))
        os.chdir(tmp_dir)
        try:
            ret.build()
        finally:
            os.chdir(curdir)

        content = load(text_file)
        assert content == msg

    def test_fuzzy_patch(self):
        conanfile = dedent("""
            from conan import ConanFile
            from conan.tools.files import patch
            import os

            class ConanFileToolsTest(ConanFile):
                name = "fuzz"
                version = "0.1.0"
                exports_sources = "*"

                def build(self):
                    patch(self, patch_file="fuzzy.patch", fuzz=True)
        """)
        source = dedent("""X
Y
Z""")
        patch = dedent("""diff --git a/Jamroot b/Jamroot
index a6981dd..0c08f09 100644
--- a/Jamroot
+++ b/Jamroot
@@ -1,3 +1,4 @@
 X
 YYYY
+V
 W""")
        expected = dedent("""X
Y
V
Z""")
        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "fuzzy.patch": patch,
                     "Jamroot": source})
        client.run("install .")
        client.run("build .")
        content = client.load("Jamroot")
        assert expected in content



import os
import pathlib
import platform
import re
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.tools.files import load


def test_bazel():
    # https://github.com/conan-io/conan/issues/10471
    dep = textwrap.dedent("""
        from conan import ConanFile
        class ExampleConanIntegration(ConanFile):
            name = "dep"
            version = "0.1"
            def package_info(self):
                self.cpp_info.includedirs = []
                self.cpp_info.libs = []
        """)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.google import BazelToolchain, BazelDeps

        class ExampleConanIntegration(ConanFile):
            generators = 'BazelDeps', 'BazelToolchain'
            requires = 'dep/0.1',
        """)
    c = TestClient()
    c.save({"dep/conanfile.py": dep,
            "consumer/conanfile.py": conanfile})
    c.run("create dep")
    c.run("install consumer")
    assert "conanfile.py: Generator 'BazelToolchain' calling 'generate()'" in c.out


def test_bazel_relative_paths():
    # https://github.com/conan-io/conan/issues/10476
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.google import BazelToolchain, BazelDeps

        class ExampleConanIntegration(ConanFile):
            generators = 'BazelDeps', 'BazelToolchain'
            requires = 'dep/0.1'

            def layout(self):
                self.folders.generators = "conandeps"
        """)
    c = TestClient()
    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
            "consumer/conanfile.py": conanfile})
    c.run("create dep")
    c.run("install consumer")
    assert "conanfile.py: Generator 'BazelToolchain' calling 'generate()'" in c.out
    build_file = c.load("consumer/conandeps/dep/BUILD.bazel")
    expected = textwrap.dedent("""\
    # Components precompiled libs
    # Root package precompiled libs

    # Components libraries declaration
    # Package library declaration
    cc_library(
        name = "dep",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
    )

    # Filegroup library declaration
    filegroup(
        name = "dep_binaries",
        srcs = glob([
            "bin/**",
        ],
        allow_empty = True
        ),
        visibility = ["//visibility:public"],
    )
    """)
    assert build_file == expected


def test_bazel_exclude_folders():
    # https://github.com/conan-io/conan/issues/11081
    dep = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save
        class ExampleConanIntegration(ConanFile):
            name = "dep"
            version = "0.1"
            def package(self):
                save(self, os.path.join(self.package_folder, "lib", "mymath", "otherfile.a"), "")
                save(self, os.path.join(self.package_folder, "lib", "libmymath.a"), "")
            def package_info(self):
                self.cpp_info.libs = ["mymath"]
        """)
    c = TestClient()
    c.save({"dep/conanfile.py": dep})
    c.run("create dep")
    c.run("install --requires=dep/0.1 -g BazelDeps")
    build_file = c.load("dep/BUILD.bazel")
    assert 'static_library = "lib/libmymath.a"' in build_file


def test_bazeldeps_and_tool_requires():
    """
    Testing that direct build requires are not included in dependencies BUILD.bazel files

    Issues related:
        * https://github.com/conan-io/conan/issues/12444
        * https://github.com/conan-io/conan/issues/12236
    """
    c = TestClient()
    tool = textwrap.dedent("""
        import os
        from conan import ConanFile

        class ToolConanfile(ConanFile):
            name = "tool"
            version = "0.1"

            def package_info(self):
                self.cpp_info.libs = ["mymath"]
        """)
    c.save({"tool/conanfile.py": tool})
    c.run("create tool")
    dep = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save
        class ExampleConanIntegration(ConanFile):
            name = "dep"
            version = "0.1"

            def build_requirements(self):
                self.tool_requires("tool/0.1")
            def package(self):
                save(self, os.path.join(self.package_folder, "lib", "mymath", "otherfile.a"), "")
                save(self, os.path.join(self.package_folder, "lib", "libmymath.a"), "")
            def package_info(self):
                self.cpp_info.libs = ["mymath"]
                self.cpp_info.bindirs = ["bin1", "bin2"]
        """)
    c.save({"dep/conanfile.py": dep})
    c.run("export dep")
    c.run("install --requires=dep/0.1 -g BazelDeps --build=missing")
    build_file = c.load("dep/BUILD.bazel")
    expected = textwrap.dedent("""\
    # Components precompiled libs
    # Root package precompiled libs
    cc_import(
        name = "mymath_precompiled",
        static_library = "lib/libmymath.a",
    )

    # Components libraries declaration
    # Package library declaration
    cc_library(
        name = "dep",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":mymath_precompiled",
        ],
    )

    # Filegroup library declaration
    filegroup(
        name = "dep_binaries",
        srcs = glob([
            "bin1/**",
            "bin2/**",
        ],
        allow_empty = True
        ),
        visibility = ["//visibility:public"],
    )
    """)
    assert expected == build_file


def test_pkg_with_public_deps_and_component_requires():
    """
    Testing a complex structure like:

    * first/0.1
        - Global bazel_target_name == "myfirstlib"
        - Components: "cmp1"
    * other/0.1
    * second/0.1
        - Requires: "first/0.1"
        - Components: "mycomponent", "myfirstcomp"
            + "mycomponent" requires "first::cmp1"
            + "myfirstcomp" requires "mycomponent"
    * third/0.1
        - Requires: "second/0.1", "other/0.1"

    Expected file structure after running BazelDeps as generator:
        - second/BUILD.bazel
        - third/BUILD.bazel
        - other/BUILD.bazel
        - myfirstlib/BUILD.bazel
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save

        class Recipe(ConanFile):

            def package(self):
                # Saving an empty lib
                dest_lib = os.path.join(self.package_folder, "lib", "libcmp1.a")
                save(self, dest_lib, "")

            def package_info(self):
                self.cpp_info.set_property("bazel_target_name", "myfirstlib")
                self.cpp_info.components["cmp1"].libs = ["libcmp1"]
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=first --version=0.1")
    client.save({"conanfile.py": GenConanfile("other", "0.1").with_package_file("file.h", "0.1")})
    client.run("create .")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgBazelConan(ConanFile):
            requires = "first/0.1"

            def package_info(self):
                self.cpp_info.components["mycomponent"].requires.append("first::cmp1")
                self.cpp_info.components["myfirstcomp"].requires.append("mycomponent")

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . --name=second --version=0.1")
    client.save({"conanfile.py": GenConanfile("third", "0.1").with_package_file("file.h", "0.1")
                                                             .with_require("second/0.1")
                                                             .with_require("other/0.1")},
                clean_first=True)
    client.run("create .")

    client2 = TestClient(cache_folder=client.cache_folder)
    conanfile = textwrap.dedent("""
        [requires]
        third/0.1

        [generators]
        BazelDeps
        """)
    client2.save({"conanfile.txt": conanfile})
    client2.run("install .")
    content = client2.load("third/BUILD.bazel")
    assert textwrap.dedent("""\
    cc_library(
        name = "third",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            "@second//:second",
            "@other//:other",
        ],
    )""") in content
    content = client2.load("second/BUILD.bazel")
    assert textwrap.dedent("""\
    # Components libraries declaration
    cc_library(
        name = "second-mycomponent",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            "@first//:myfirstlib-cmp1",
        ],
    )

    cc_library(
        name = "second-myfirstcomp",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":second-mycomponent",
        ],
    )

    # Package library declaration
    cc_library(
        name = "second",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":second-mycomponent",
            ":second-myfirstcomp",
            "@first//:myfirstlib",
        ],
    )""") in content
    content = client2.load("first/BUILD.bazel")
    assert textwrap.dedent("""\
    # Components precompiled libs
    cc_import(
        name = "libcmp1_precompiled",
        static_library = "lib/libcmp1.a",
    )

    # Root package precompiled libs

    # Components libraries declaration
    cc_library(
        name = "myfirstlib-cmp1",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":libcmp1_precompiled",
        ],
    )

    # Package library declaration
    cc_library(
        name = "myfirstlib",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":myfirstlib-cmp1",
        ],
    )""") in content
    content = client2.load("other/BUILD.bazel")
    assert "deps =" not in content


def test_pkg_with_public_deps_and_component_requires_2():
    """
    Testing another complex structure like:

    * other/0.1
        - Global bazel_target_name == "fancy_name"
        - Components: "cmp1", "cmp2", "cmp3"
            + "cmp1" bazel_target_name == "component1" (it shouldn't be affected by "fancy_name")
            + "cmp3" bazel_target_name == "component3" (it shouldn't be affected by "fancy_name")
            + "cmp3" requires "cmp1"
    * pkg/0.1
        - Requires: "other/0.1" -> "other::cmp1"

    Expected file structure after running BazelDeps as generator:
        - fancy_name/BUILD.bazel (components: "component1", "fancy_name-cmp2", "component3")
        - pkg/BUILD.bazel
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save

        class Recipe(ConanFile):

            def package(self):
                # Saving an empty lib
                dest_lib = os.path.join(self.package_folder, "lib", "libother_cmp1.a")
                dest_lib2 = os.path.join(self.package_folder, "lib", "libother_cmp2.a")
                save(self, dest_lib, "")
                save(self, dest_lib2, "")

            def package_info(self):
                self.cpp_info.set_property("bazel_target_name", "fancy_name")
                self.cpp_info.components["cmp1"].libs = ["other_cmp1"]
                self.cpp_info.components["cmp1"].set_property("bazel_target_name", "component1")
                self.cpp_info.components["cmp2"].libs = ["other_cmp2"]
                self.cpp_info.components["cmp3"].requires.append("cmp1")
                self.cpp_info.components["cmp3"].set_property("bazel_target_name", "component3")
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=other --version=1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgBazelConan(ConanFile):
            requires = "other/1.0"

            def package_info(self):
                self.cpp_info.requires = ["other::cmp1"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=pkg --version=0.1")

    client2 = TestClient(cache_folder=client.cache_folder)
    conanfile = textwrap.dedent("""
        [requires]
        pkg/0.1

        [generators]
        BazelDeps
        """)
    client2.save({"conanfile.txt": conanfile})
    client2.run("install .")
    content = client2.load("pkg/BUILD.bazel")
    assert textwrap.dedent("""\
    # Package library declaration
    cc_library(
        name = "pkg",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            "@other//:component1",
        ],
    )""") in content
    content = client2.load("other/BUILD.bazel")
    assert textwrap.dedent("""\
    # Components precompiled libs
    cc_import(
        name = "other_cmp1_precompiled",
        static_library = "lib/libother_cmp1.a",
    )

    cc_import(
        name = "other_cmp2_precompiled",
        static_library = "lib/libother_cmp2.a",
    )


    # Root package precompiled libs

    # Components libraries declaration
    cc_library(
        name = "component1",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":other_cmp1_precompiled",
        ],
    )

    cc_library(
        name = "fancy_name-cmp2",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":other_cmp2_precompiled",
        ],
    )

    cc_library(
        name = "component3",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":component1",
        ],
    )

    # Package library declaration
    cc_library(
        name = "fancy_name",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":component1",
            ":fancy_name-cmp2",
            ":component3",
        ],
    )""") in content


def test_pkgconfigdeps_with_test_requires():
    """
    BazelDeps has to create any declared test requirements on the recipe.
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save

        class Recipe(ConanFile):

            def package(self):
                # Saving an empty lib
                dest_lib = os.path.join(self.package_folder, "lib", "liblib{0}.a")
                save(self, dest_lib, "")

            def package_info(self):
                self.cpp_info.libs = ["lib{0}"]
        """)
    with client.chdir("app"):
        client.save({"conanfile.py": conanfile.format("app")})
        client.run("create . --name=app --version=1.0")
    with client.chdir("test"):
        client.save({"conanfile.py": conanfile.format("test")})
        client.run("create . --name=test --version=1.0")
    # Create library having build and test requires
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class HelloLib(ConanFile):
            def build_requirements(self):
                self.test_requires('app/1.0')
                self.test_requires('test/1.0')
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -g BazelDeps")
    expected = textwrap.dedent("""\
    # Root package precompiled libs
    cc_import(
        name = "lib{0}_precompiled",
        static_library = "lib/liblib{0}.a",
    )

    # Components libraries declaration
    # Package library declaration
    cc_library(
        name = "{0}",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":lib{0}_precompiled",
        ],
    )""")
    assert expected.format("test") in client.load("test/BUILD.bazel")
    assert expected.format("app") in client.load("app/BUILD.bazel")


def test_with_editable_layout():
    """
    Testing if editable mode is working for a Bazel project (using BazelDeps and bazel_layout)
    """
    client = TestClient()
    dep = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.google import bazel_layout
        from conan.tools.files import save
        class Dep(ConanFile):
            name = "dep"
            version = "0.1"

            def layout(self):
                bazel_layout(self, target_folder="main")
                self.cpp.source.includedirs = ["include"]

            def package_info(self):
                self.cpp_info.libs = ["mylib"]
        """)
    client.save({"dep/conanfile.py": dep,
                 "dep/include/header.h": "",
                 "dep/bazel-bin/main/libmylib.a": "",
                 "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_requires("dep/0.1")})
    client.run("create dep")
    client.run("editable add dep --name=dep --version=0.1")
    recipes_folder = client.current_folder.replace("\\", "/")
    with client.chdir("pkg"):
        client.run("install . -g BazelDeps")
        # TODO: Remove when dropped Bazel 6.x compatibility
        content = client.load("dependencies.bzl")
        assert textwrap.dedent(f"""\
        def load_conan_dependencies():
            native.new_local_repository(
                name="dep",
                path="{recipes_folder}/dep",
                build_file="{recipes_folder}/pkg/dep/BUILD.bazel",
            )""") in content
        # Bazel 7.x
        content = client.load("conan_deps_module_extension.bzl")
        assert textwrap.dedent(f"""\
        def _load_dependencies_impl(mctx):
            conan_dependency_repo(
                name = "dep",
                package_path = "{recipes_folder}/dep",
                build_file_path = "{recipes_folder}/pkg/dep/BUILD.bazel",
            )""") in content
        content = client.load("dep/BUILD.bazel")
        assert pathlib.Path(client.current_folder, "conan_deps_repo_rules.bzl").exists()
        assert textwrap.dedent("""\
        cc_import(
            name = "mylib_precompiled",
            static_library = "bazel-bin/main/libmylib.a",
        )

        # Components libraries declaration
        # Package library declaration
        cc_library(
            name = "dep",
            hdrs = glob([
                "include/**",
            ],
            allow_empty = True
            ),
            includes = [
                "include",
            ],
            visibility = ["//visibility:public"],
            deps = [
                # do not sort
                ":mylib_precompiled",
            ],
        )""") in content


def test_tool_requires():
    """
    Testing if BUILD files are created for tool requires if build_context_activated=True is used.
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save

        class PkgBazelConan(ConanFile):

            def package(self):
                import os
                libdirs = os.path.join(self.package_folder, "lib")
                save(self, os.path.join(libdirs, "libtool.lib"), "")

            def package_info(self):
                self.cpp_info.libs = ["libtool"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=tool --version=1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save

        class PkgBazelConan(ConanFile):

            def package(self):
                import os
                libdirs = os.path.join(self.package_folder, "lib")
                save(self, os.path.join(libdirs, "other_cmp1.lib"), "")
                save(self, os.path.join(libdirs, "other_cmp2.lib"), "")

            def package_info(self):
                self.cpp_info.set_property("bazel_target_name", "libother")
                self.cpp_info.set_property("bazel_repository_name", "other-repo")
                self.cpp_info.components["cmp1"].libs = ["other_cmp1"]
                self.cpp_info.components["cmp1"].set_property("bazel_target_name", "component1")
                self.cpp_info.components["cmp2"].libs = ["other_cmp2"]
                self.cpp_info.components["cmp3"].requires.append("cmp1")
                self.cpp_info.components["cmp3"].set_property("bazel_target_name", "component3")
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . --name=other --version=1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.google import BazelDeps

        class PkgBazelConan(ConanFile):
            name = "demo"
            version = "1.0"

            def build_requirements(self):
                self.tool_requires("tool/1.0")
                self.tool_requires("other/1.0")

            def generate(self):
                tc = BazelDeps(self)
                tc.build_context_activated = ["other", "tool"]
                tc.generate()
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -pr:h default -pr:b default")
    assert 'name = "tool"' in client.load("build-tool/BUILD.bazel")
    assert textwrap.dedent("""\
    # Components precompiled libs
    cc_import(
        name = "other_cmp1_precompiled",
        static_library = "lib/other_cmp1.lib",
    )

    cc_import(
        name = "other_cmp2_precompiled",
        static_library = "lib/other_cmp2.lib",
    )


    # Root package precompiled libs

    # Components libraries declaration
    cc_library(
        name = "component1",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":other_cmp1_precompiled",
        ],
    )

    cc_library(
        name = "libother-cmp2",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":other_cmp2_precompiled",
        ],
    )

    cc_library(
        name = "component3",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":component1",
        ],
    )

    # Package library declaration
    cc_library(
        name = "libother",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":component1",
            ":libother-cmp2",
            ":component3",
        ],
    )""") in client.load("build-other-repo/BUILD.bazel")
    # TODO: Remove when dropped Bazel 6.x compatibility
    # Let's check if the names used in the dependencies.bzl are correct
    content = client.load("dependencies.bzl")
    assert 'name="build-other-repo"' in content  # build context + bazel_repository_name prop
    assert 'name="build-tool"' in content  # build context + package reference name
    # Bazel 7.x
    # Let's check if the names used in the conan_deps_repo_rules.bzl are correct
    content = client.load("conan_deps_module_extension.bzl")
    assert 'name = "build-other-repo"' in content  # build context + bazel_repository_name prop
    assert 'name = "build-tool"' in content  # build context + package reference name


def test_tool_requires_not_created_if_no_activated():
    """
    Testing if there are no BUILD files created in no context are activated
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save

        class PkgBazelConan(ConanFile):

            def package(self):
                import os
                libdirs = os.path.join(self.package_folder, "lib")
                save(self, os.path.join(libdirs, "libtool.lib"), "")
            def package_info(self):
                self.cpp_info.libs = ["libtool"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=tool --version=1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgBazelConan(ConanFile):
            name = "demo"
            version = "1.0"
            generators = "BazelDeps"

            def build_requirements(self):
                self.tool_requires("tool/1.0")

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -pr:h default -pr:b default")
    assert not os.path.exists(os.path.join(client.current_folder, "tool"))


def test_tool_requires_raise_exception_if_exist_both_require_and_build_one():
    """
    Testing if same dependency exists in both host and build context
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save

        class PkgBazelConan(ConanFile):

            def package(self):
                import os
                libdirs = os.path.join(self.package_folder, "lib")
                save(self, os.path.join(libdirs, "libtool.lib"), "")
            def package_info(self):
                self.cpp_info.libs = ["libtool"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=tool --version=1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.google import BazelDeps

        class PkgBazelConan(ConanFile):
            name = "demo"
            version = "1.0"

            def requirements(self):
                self.requires("tool/1.0")

            def build_requirements(self):
                self.tool_requires("tool/1.0")

            def generate(self):
                tc = BazelDeps(self)
                tc.build_context_activated = ["tool"]
                tc.generate()
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -pr:h default -pr:b default")
    # Different Bazel repository names, same target names
    assert 'name = "tool"' in client.load("build-tool/BUILD.bazel")
    assert 'name = "tool"' in client.load("tool/BUILD.bazel")


def test_error_missing_bazel_build_files_in_build_context():
    """
    BazelDeps should generate BUILD files in the build context for a
    test_package that both requires(example/1.0) and tool_requires(example/1.0),
    which depends on other requirements
    * Issue related: https://github.com/conan-io/conan/issues/12664
    """
    c = TestClient()
    example = textwrap.dedent("""
        import os
        from conan import ConanFile
        class Example(ConanFile):
            name = "example"
            version = "1.0"
            requires = "game/1.0"
            generators = "BazelDeps"
            settings = "build_type"
            def build(self):
                context = "build-" if self.context == "build" else ""
                assert os.path.exists(os.path.join(f"{context}math", "BUILD.bazel"))
                assert os.path.exists(os.path.join(f"{context}engine", "BUILD.bazel"))
                assert os.path.exists(os.path.join(f"{context}game", "BUILD.bazel"))
            """)
    c.save({"math/conanfile.py": GenConanfile("math", "1.0").with_settings("build_type"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_settings("build_type")
                                                                .with_require("math/1.0"),
            "game/conanfile.py": GenConanfile("game", "1.0").with_settings("build_type")
                                                            .with_requires("engine/1.0"),
            "example/conanfile.py": example,
            # With ``with_test()`` it already generates a requires(example/1.0)
            "example/test_package/conanfile.py": GenConanfile().with_build_requires("example/1.0")
                                                               .with_test("pass")})
    c.run("create math")
    c.run("create engine")
    c.run("create game")
    # This used to crash because of the assert inside the build() method
    c.run("create example -pr:b=default -pr:h=default")
    # Now make sure we can actually build with build!=host context
    # The debug binaries are missing, so adding --build=missing
    c.run("create example -pr:b=default -pr:h=default -s:h build_type=Debug --build=missing "
          "--build=example")
    c.assert_listed_require({"example/1.0": "Cache"})
    c.assert_listed_require({"example/1.0": "Cache"}, build=True)


class TestBazelGenerationBuildContext:
    """
    https://github.com/conan-io/conan/issues/15764
    """
    def test_bazel_generate(self):
        c = TestClient()
        tool = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.google import BazelDeps
            class Example(ConanFile):
                name = "tool"
                version = "1.0"
                requires = "wayland/1.0"
                tool_requires = "wayland/1.0"
                def generate(self):
                    tc = BazelDeps(self)
                    tc.build_context_activated = ["wayland", "dep"]
                    tc.generate()
                def build(self):
                    # Build context
                    assert os.path.exists(os.path.join("build-wayland", "BUILD.bazel"))
                    # Revisit this after investigation -> build_req = conanfile.dependencies.direct_build
                    # assert os.path.exists(os.path.join("build-dep", "BUILD.bazel"))
                    # Host context
                    assert os.path.exists(os.path.join("wayland", "BUILD.bazel"))
                    assert os.path.exists(os.path.join("dep", "BUILD.bazel"))
                """)
        c.save({"dep/conanfile.py": GenConanfile("dep", "1.0").with_package_type("shared-library"),
                "wayland/conanfile.py": GenConanfile("wayland", "1.0").with_requires("dep/1.0"),
                "tool/conanfile.py": tool,
                "app/conanfile.py": GenConanfile().with_tool_requires("tool/1.0")})
        c.run("export dep")
        c.run("export wayland")
        c.run("export tool")
        c.run("install app --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail
        # Now make sure we can actually build with build!=host context
        c.run("install app -s:h build_type=Debug --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail

    def test_bazel_generate_components(self):
        c = TestClient()
        tool = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.google import BazelDeps
            class Example(ConanFile):
                name = "tool"
                version = "1.0"
                requires = "wayland/1.0"
                tool_requires = "wayland/1.0"
                def generate(self):
                    tc = BazelDeps(self)
                    tc.build_context_activated = ["wayland", "dep"]
                    tc.generate()
                def build(self):
                    # Build context
                    assert os.path.exists(os.path.join("build-wayland", "BUILD.bazel"))
                    # Revisit this after investigation -> build_req = conanfile.dependencies.direct_build
                    # assert os.path.exists(os.path.join("build-dep", "BUILD.bazel"))
                    # Host context
                    assert os.path.exists(os.path.join("wayland", "BUILD.bazel"))
                    assert os.path.exists(os.path.join("dep", "BUILD.bazel"))
                """)
        wayland = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "wayland"
                version = "1.0"
                requires = "dep/1.0"
                def package_info(self):
                    self.cpp_info.components["client"].libs = []
                    self.cpp_info.components["server"].libs = []
            """)
        c.save({"dep/conanfile.py": GenConanfile("dep", "1.0").with_package_type("shared-library"),
                "wayland/conanfile.py": wayland,
                "tool/conanfile.py": tool,
                "app/conanfile.py": GenConanfile().with_tool_requires("tool/1.0")})
        c.run("export dep")
        c.run("export wayland")
        c.run("export tool")
        c.run("install app --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail
        # Now make sure we can actually build with build!=host context
        c.run("install app -s:h build_type=Debug --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail



def test_shared_windows_find_libraries():
    """
    Testing the ``_get_libs`` mechanism in Windows, the shared libraries and their
    import ones are correctly found.

    Note: simulating dependencies with openssl, libcurl, and zlib packages:

    zlib:
        - (zlib package) bin/zlib1.dll AND lib/zdll.lib
    libcurl:
        - (curl component) bin/libcurl.dll AND lib/libcurl_imp.lib
    openssl:
        - (crypto component) bin/libcrypto-3-x64.dll AND lib/libcrypto.lib
        - (ssl component) bin/libssl-3-x64.dll AND lib/libssl.lib


    Issue: https://github.com/conan-io/conan/issues/16691
    """
    c = TestClient()
    zlib = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save
        class Example(ConanFile):
            name = "zlib"
            version = "1.0"
            options = {"shared": [True, False]}
            default_options = {"shared": False}
            def package(self):
                bindirs = os.path.join(self.package_folder, "bin")
                libdirs = os.path.join(self.package_folder, "lib")
                save(self, os.path.join(bindirs, "zlib1.dll"), "")
                save(self, os.path.join(libdirs, "zdll.lib"), "")
            def package_info(self):
                self.cpp_info.libs = ["zdll"]
            """)
    libiconv = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save
        class Example(ConanFile):
            name = "libiconv"
            version = "1.0"
            options = {"shared": [True, False]}
            default_options = {"shared": False}
            def package(self):
                bindirs = os.path.join(self.package_folder, "bin")
                libdirs = os.path.join(self.package_folder, "lib")
                save(self, os.path.join(bindirs, "charset-1.dll"), "")
                save(self, os.path.join(bindirs, "iconv-2.dll"), "")
                save(self, os.path.join(libdirs, "charset.lib"), "")
                save(self, os.path.join(libdirs, "iconv.lib"), "")
            def package_info(self):
                self.cpp_info.libs = ["iconv", "charset"]
            """)
    openssl = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save
        class Pkg(ConanFile):
            name = "openssl"
            version = "1.0"
            options = {"shared": [True, False]}
            default_options = {"shared": False}
            def package(self):
                bindirs = os.path.join(self.package_folder, "bin")
                libdirs = os.path.join(self.package_folder, "lib")
                save(self, os.path.join(bindirs, "libcrypto-3-x64.dll"), "")
                save(self, os.path.join(bindirs, "libssl-3-x64.dll"), "")
                save(self, os.path.join(libdirs, "libcrypto.lib"), "")
                save(self, os.path.join(libdirs, "libssl.lib"), "")
            def package_info(self):
                self.cpp_info.components["crypto"].libs = ["libcrypto"]
                self.cpp_info.components["ssl"].libs = ["libssl"]
        """)
    libcurl = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save
        class Example(ConanFile):
            name = "libcurl"
            version = "1.0"
            options = {"shared": [True, False]}
            default_options = {"shared": False}
            def package(self):
                bindirs = os.path.join(self.package_folder, "bin")
                libdirs = os.path.join(self.package_folder, "lib")
                save(self, os.path.join(bindirs, "libcurl.dll"), "")
                save(self, os.path.join(libdirs, "libcurl_imp.lib"), "")
            def package_info(self):
                self.cpp_info.components["curl"].libs = ["libcurl_imp"]
            """)
    consumer = textwrap.dedent("""
        [requires]
        zlib/1.0
        openssl/1.0
        libcurl/1.0
        libiconv/1.0
        [options]
        *:shared=True
    """)
    c.save({"conanfile.txt": consumer,
            "zlib/conanfile.py": zlib,
            "openssl/conanfile.py": openssl,
            "libcurl/conanfile.py": libcurl,
            "libiconv/conanfile.py": libiconv,
    })
    c.run("export-pkg zlib -o:a shared=True")
    c.run("export-pkg openssl -o:a shared=True")
    c.run("export-pkg libcurl -o:a shared=True")
    c.run("export-pkg libiconv -o:a shared=True")
    c.run("install . -g BazelDeps")
    libcurl_bazel_build = load(None, os.path.join(c.current_folder, "libcurl", "BUILD.bazel"))
    zlib_bazel_build = load(None, os.path.join(c.current_folder, "zlib", "BUILD.bazel"))
    openssl_bazel_build = load(None, os.path.join(c.current_folder, "openssl", "BUILD.bazel"))
    libiconv_bazel_build = load(None, os.path.join(c.current_folder, "libiconv", "BUILD.bazel"))
    libcurl_expected = textwrap.dedent("""\
    # Components precompiled libs
    cc_import(
        name = "libcurl_imp_precompiled",
        shared_library = "bin/libcurl.dll",
        interface_library = "lib/libcurl_imp.lib",
    )
    """)
    openssl_expected = textwrap.dedent("""\
    # Components precompiled libs
    cc_import(
        name = "libcrypto_precompiled",
        shared_library = "bin/libcrypto-3-x64.dll",
        interface_library = "lib/libcrypto.lib",
    )

    cc_import(
        name = "libssl_precompiled",
        shared_library = "bin/libssl-3-x64.dll",
        interface_library = "lib/libssl.lib",
    )
    """)
    zlib_expected = textwrap.dedent("""\
    cc_import(
        name = "zdll_precompiled",
        shared_library = "bin/zlib1.dll",
        interface_library = "lib/zdll.lib",
    )
    """)
    iconv_expected = textwrap.dedent("""\
    cc_import(
        name = "iconv_precompiled",
        shared_library = "bin/iconv-2.dll",
        interface_library = "lib/iconv.lib",
    )
    """)
    charset_expected = textwrap.dedent("""\
    cc_import(
        name = "charset_precompiled",
        shared_library = "bin/charset-1.dll",
        interface_library = "lib/charset.lib",
    )
    """)
    assert libcurl_expected in libcurl_bazel_build
    assert zlib_expected in zlib_bazel_build
    assert openssl_expected in openssl_bazel_build
    assert iconv_expected in libiconv_bazel_build and charset_expected in libiconv_bazel_build


def test_pkg_with_duplicated_component_requires():
    """
    Testing that even having duplicated component requires, the PC does not include them.
    Issue: https://github.com/conan-io/conan/issues/18283
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            def package_info(self):
                self.cpp_info.components["mycomponent"].libs = []
                self.cpp_info.components["myfirstcomp"].requires.append("mycomponent")
                # Duplicate one
                self.cpp_info.components["myfirstcomp"].requires.append("mycomponent")

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . --name=mylib --version=0.1")
    client.save({"conanfile.py": GenConanfile("pkg", "0.1").with_require("mylib/0.1")},
                clean_first=True)
    client.run("install . -g BazelDeps")
    build_content = load(None, os.path.join(client.current_folder, "mylib", "BUILD.bazel"))
    myfirstcomp_expected = textwrap.dedent("""\
    cc_library(
        name = "mylib-myfirstcomp",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        visibility = ["//visibility:public"],
        deps = [
            # do not sort
            ":mylib-mycomponent",
        ],
    )
    """)
    assert myfirstcomp_expected in build_content


@pytest.mark.skipif(platform.system() == "Windows", reason="Unix paths only")
def test_apple_frameworks_and_frameworkdirs():
    """
    Testing that Apple frameworks are included as linkopts
    Issue: https://github.com/conan-io/conan/issues/18748
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            def package_info(self):
                self.cpp_info.frameworks = ["CoreFoundation", "Cocoa"]
                self.cpp_info.frameworkdirs = ["/my/path/to/frw1"]

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . --name=mylib --version=0.1")
    client.save({"conanfile.py": GenConanfile("pkg", "0.1").with_require("mylib/0.1")},
                clean_first=True)
    client.run("install . -g BazelDeps")
    build_content = load(None, os.path.join(client.current_folder, "mylib", "BUILD.bazel"))
    build_file_expected = textwrap.dedent("""\
    cc_library(
        name = "mylib",
        hdrs = glob([
            "include/**",
        ],
        allow_empty = True
        ),
        includes = [
            "include",
        ],
        linkopts = [
            "-framework",
            "CoreFoundation",
            "-framework",
            "Cocoa",
            "-F",
            "/my/path/to/frw1",
        ],
        visibility = ["//visibility:public"],
    )
    """)
    assert build_file_expected in build_content


def test_shared_libs_and_unix_includes_rpath():
    """
    Testing the RPATH flag is added to the BUILD.bazel file if these conditions
    are given: shared library and UNIX systems.

    Issue: https://github.com/conan-io/conan/issues/19190
    Issue: https://github.com/conan-io/conan/issues/19135
    """
    client = TestClient()
    csm = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save
        class Pkg(ConanFile):
            name = "csm"
            version = "1.0"
            settings = "os"
            options = {"shared": [True, False]}
            default_options = {"shared": False}
            def package(self):
                if self.settings.os == "Windows":
                    save(self, os.path.join(self.package_folder, "bin", "csmapi.dll"), "")
                    save(self, os.path.join(self.package_folder, "lib", "csmapi.lib"), "")
                else:
                    save(self, os.path.join(self.package_folder, "lib", "libcsmapi.so"), "")
            def package_info(self):
                self.cpp_info.libs = ["csmapi"]
        """)
    consumer = textwrap.dedent("""
        [requires]
        csm/1.0
        [options]
        *:shared=True
    """)
    client.save({"conanfile.txt": consumer, "csm/conanfile.py": csm})
    client.run("export-pkg csm -o '*:shared=True' -s 'os=Windows'")
    client.run("export-pkg csm -o '*:shared=True' -s 'os=Linux'")
    client.run("install . -g BazelDeps -s 'os=Windows'")
    build_content = load(None, os.path.join(client.current_folder, "csm", "BUILD.bazel"))
    assert '"-Wl,-rpath,' not in build_content
    client.run("install . -g BazelDeps -s 'os=Linux'")
    build_content = load(None, os.path.join(client.current_folder, "csm", "BUILD.bazel"))
    assert re.search('"-Wl,-rpath,.*/p/lib"', build_content.replace("\\", "/"))



import os
import platform
import textwrap

import pytest

from conan.tools.apple.apple import _to_apple_arch
from conan.test.assets.autotools import gen_makefile
from conan.test.assets.sources import gen_function_h, gen_function_cpp
from conan.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.parametrize("config", [("x86_64", "Macos", "10.14", None),
                                    ("armv8", "iOS", "10.0", "iphoneos"),
                                    ("x86_64", "iOS", "10.0", "iphonesimulator"),
                                    ("armv8", "Macos", "10.14", None)  # M1
                                    ])
def test_makefile_arch(config):
    makefile = gen_makefile(apps=["app"], libs=["hello"])
    conanfile_py = textwrap.dedent("""
        from conan import ConanFile, tools
        from conan.tools.gnu import Autotools

        class App(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            options = {"shared": [True, False], "fPIC": [True, False]}
            default_options = {"shared": False, "fPIC": True}
            generators = "GnuToolchain"

            def config_options(self):
                if self.settings.os == "Windows":
                    self.options.rm_safe("fPIC")

            def configure(self):
                if self.options.shared:
                    self.options.rm_safe("fPIC")

            def build(self):
                env_build = Autotools(self)
                env_build.make()
    """)
    arch, os_, os_version, os_sdk = config
    profile = textwrap.dedent("""
                include(default)
                [settings]
                os = {os}
                {os_sdk}
                os.version = {os_version}
                arch = {arch}
                """).format(os=os_, arch=arch,
                            os_version=os_version, os_sdk="os.sdk = " + os_sdk if os_sdk else "")

    t = TestClient()
    hello_h = gen_function_h(name="hello")
    hello_cpp = gen_function_cpp(name="hello")
    main_cpp = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])

    t.save({"Makefile": makefile,
            "hello.h": hello_h,
            "hello.cpp": hello_cpp,
            "app.cpp": main_cpp,
            "conanfile.py": conanfile_py,
            "profile": profile})

    t.run("install . --profile:host=profile --profile:build=default")
    t.run("build . --profile:host=profile --profile:build=default")

    libhello = os.path.join(t.current_folder, "libhello.a")
    app = os.path.join(t.current_folder, "app")
    assert os.path.isfile(libhello)
    assert os.path.isfile(app)

    expected_arch = _to_apple_arch(arch)
    t.run_command('lipo -info "%s"' % libhello)
    assert "architecture: %s" % expected_arch in t.out

    t.run_command('lipo -info "%s"' % app)
    assert "architecture: %s" % expected_arch in t.out



import os
import textwrap
from collections import OrderedDict

from jinja2 import Template

from conan.api.output import ConanOutput
from conan.internal import check_duplicated_generator
from conan.tools.build import use_win_mingw
from conan.tools.cmake.presets import write_cmake_presets
from conan.tools.cmake.toolchain import CONAN_TOOLCHAIN_FILENAME
from conan.tools.cmake.toolchain.blocks import (ExtraVariablesBlock, ToolchainBlocks,
                                                UserToolchain, GenericSystemBlock,
                                                AndroidSystemBlock, AppleSystemBlock, FPicBlock,
                                                ArchitectureBlock, GLibCXXBlock, VSRuntimeBlock,
                                                CppStdBlock, ParallelBlock, CMakeFlagsInitBlock,
                                                TryCompileBlock, FindFiles, PkgConfigBlock,
                                                SkipRPath, SharedLibBock, OutputDirsBlock,
                                                ExtraFlagsBlock, CompilersBlock, LinkerScriptsBlock,
                                                VSDebuggerEnvironment, VariablesBlock,
                                                PreprocessorBlock)
from conan.tools.cmake.utils import is_multi_configuration
from conan.tools.env import VirtualBuildEnv, VirtualRunEnv
from conan.tools.intel import IntelCC
from conan.tools.microsoft import VCVars
from conan.tools.microsoft.visual import vs_ide_version
from conan.errors import ConanException
from conan.internal.model.options import _PackageOption
from conan.internal.graph.graph import RECIPE_CONSUMER, RECIPE_EDITABLE
from conan.internal.util.files import save


class Variables(OrderedDict):
    _configuration_types = None  # Needed for py27 to avoid infinite recursion

    def __init__(self):
        super(Variables, self).__init__()
        self._configuration_types = {}

    def __getattribute__(self, config):
        try:
            return super(Variables, self).__getattribute__(config)
        except AttributeError:
            return self._configuration_types.setdefault(config, OrderedDict())

    @property
    def configuration_types(self):
        # Reverse index for the configuration_types variables
        ret = OrderedDict()
        for conf, definitions in self._configuration_types.items():
            for k, v in definitions.items():
                ret.setdefault(k, []).append((conf, v))
        return ret

    def quote_preprocessor_strings(self):
        for key, var in self.items():
            if isinstance(var, str):
                self[key] = str(var).replace('"', '\\"')
        for config, data in self._configuration_types.items():
            for key, var in data.items():
                if isinstance(var, str):
                    data[key] = str(var).replace('"', '\\"')


class CMakeToolchain:

    filename = CONAN_TOOLCHAIN_FILENAME

    _template = textwrap.dedent("""\
        # Conan automatically generated toolchain file
        # DO NOT EDIT MANUALLY, it will be overwritten

        # Avoid including toolchain file several times (bad if appending to variables like
        #   CMAKE_CXX_FLAGS. See https://github.com/android/ndk/issues/323
        include_guard()
        message(STATUS "Using Conan toolchain: ${CMAKE_CURRENT_LIST_FILE}")
        if(${CMAKE_VERSION} VERSION_LESS "3.15")
            message(FATAL_ERROR "The 'CMakeToolchain' generator only works with CMake >= 3.15")
        endif()

        {% for conan_block in conan_blocks %}
        {{ conan_block }}
        {% endfor %}

        if(CMAKE_POLICY_DEFAULT_CMP0091)  # Avoid unused and not-initialized warnings
        endif()
        """)

    def __init__(self, conanfile, generator=None):
        self._conanfile = conanfile
        self.generator = self._get_generator(generator)
        self.variables = Variables()
        # This doesn't support multi-config, they go to the same configPreset common in multi-config
        self.cache_variables = {}
        self.preprocessor_definitions = Variables()

        self.extra_cxxflags = []
        self.extra_cflags = []
        self.extra_sharedlinkflags = []
        self.extra_exelinkflags = []

        self.blocks = ToolchainBlocks(self._conanfile, self,
                                      [("user_toolchain", UserToolchain),
                                       ("generic_system", GenericSystemBlock),
                                       ("compilers", CompilersBlock),
                                       ("android_system", AndroidSystemBlock),
                                       ("apple_system", AppleSystemBlock),
                                       ("fpic", FPicBlock),
                                       ("arch_flags", ArchitectureBlock),
                                       ("linker_scripts", LinkerScriptsBlock),
                                       ("libcxx", GLibCXXBlock),
                                       ("vs_runtime", VSRuntimeBlock),
                                       ("vs_debugger_environment", VSDebuggerEnvironment),
                                       ("cppstd", CppStdBlock),
                                       ("parallel", ParallelBlock),
                                       ("extra_flags", ExtraFlagsBlock),
                                       ("cmake_flags_init", CMakeFlagsInitBlock),
                                       ("extra_variables", ExtraVariablesBlock),
                                       ("try_compile", TryCompileBlock),
                                       ("find_paths", FindFiles),
                                       ("pkg_config", PkgConfigBlock),
                                       ("rpath", SkipRPath),
                                       ("shared", SharedLibBock),
                                       ("output_dirs", OutputDirsBlock),
                                       ("variables", VariablesBlock),
                                       ("preprocessor", PreprocessorBlock)])

        # Set the CMAKE_MODULE_PATH and CMAKE_PREFIX_PATH to the deps .builddirs
        self.find_builddirs = True
        self.user_presets_path = "CMakeUserPresets.json"
        self.presets_prefix = "conan"
        self.presets_build_environment = None
        self.presets_run_environment = None
        self.absolute_paths = False  # By default use relative paths to toolchain and presets

    def _context(self):
        """ Returns dict, the context for the template
        """
        self.preprocessor_definitions.quote_preprocessor_strings()

        blocks = self.blocks.process_blocks()
        ctxt_toolchain = {

            "conan_blocks": blocks
        }

        return ctxt_toolchain

    @property
    def content(self):
        context = self._context()
        content = Template(self._template, trim_blocks=True, lstrip_blocks=True,
                           keep_trailing_newline=True).render(**context)
        return content

    @property
    def is_multi_configuration(self):
        return is_multi_configuration(self.generator)

    def _find_cmake_exe(self):
        for req in self._conanfile.dependencies.direct_build.values():
            if req.ref.name == "cmake":
                for bindir in req.cpp_info.bindirs:
                    cmake_path = os.path.join(bindir, "cmake")
                    cmake_exe_path = os.path.join(bindir, "cmake.exe")

                    if os.path.exists(cmake_path):
                        return cmake_path
                    elif os.path.exists(cmake_exe_path):
                        return cmake_exe_path

    def generate(self):
        """
          This method will save the generated files to the conanfile.generators_folder
        """
        check_duplicated_generator(self, self._conanfile)
        toolchain_file = self._conanfile.conf.get("tools.cmake.cmaketoolchain:toolchain_file")
        if toolchain_file is None:  # The main toolchain file generated only if user dont define
            toolchain_file = self.filename
            save(os.path.join(self._conanfile.generators_folder, toolchain_file), self.content)
            ConanOutput(str(self._conanfile)).info(f"CMakeToolchain generated: {toolchain_file}")
        # If we're using Intel oneAPI, we need to generate the environment file and run it
        if self._conanfile.settings.get_safe("compiler") == "intel-cc":
            IntelCC(self._conanfile).generate()
        # Generators like Ninja or NMake requires an active vcvars
        elif self.generator is not None and "Visual" not in self.generator:
            VCVars(self._conanfile).generate()

        cache_variables = {}
        for name, value in self.cache_variables.items():
            if isinstance(value, bool):
                cache_variables[name] = "ON" if value else "OFF"
            elif isinstance(value, _PackageOption):
                if str(value).lower() in ["true", "false", "none"]:
                    cache_variables[name] = "ON" if bool(value) else "OFF"
                elif str(value).isdigit():
                    cache_variables[name] = int(value)
                else:
                    cache_variables[name] = str(value)
            else:
                cache_variables[name] = value

        buildenv, runenv, cmake_executable = None, None, None

        if self._conanfile.conf.get("tools.cmake.cmaketoolchain:presets_environment", default="",
                                    check_type=str, choices=("disabled", "")) != "disabled":

            build_env = self.presets_build_environment.vars(self._conanfile) \
                if self.presets_build_environment \
                else VirtualBuildEnv(self._conanfile, auto_generate=True).vars()
            run_env = self.presets_run_environment.vars(self._conanfile) \
                if self.presets_run_environment \
                else VirtualRunEnv(self._conanfile, auto_generate=True).vars()

            buildenv = {name: value for name, value in
                        build_env.items(variable_reference="$penv{{{name}}}")}
            runenv = {name: value for name, value in
                      run_env.items(variable_reference="$penv{{{name}}}")}

            cmake_executable = self._conanfile.conf.get("tools.cmake:cmake_program", None)
            cmake_executable = cmake_executable or self._find_cmake_exe()

        user_presets = self.user_presets_path
        try:  # TODO: Refactor this repeated pattern to deduce "is-consumer"
            # The user conf user_presets ONLY applies to dev space, not in the cache
            if self._conanfile._conan_node.recipe in (RECIPE_CONSUMER, RECIPE_EDITABLE):
                user_presets = self._conanfile.conf.get("tools.cmake.cmaketoolchain:user_presets",
                                                        default=self.user_presets_path)
        except AttributeError:
            pass

        write_cmake_presets(self._conanfile, toolchain_file, self.generator, cache_variables,
                            user_presets, self.presets_prefix, buildenv, runenv,
                            cmake_executable, self.absolute_paths)

    def _get_generator(self, recipe_generator):
        # Returns the name of the generator to be used by CMake
        conanfile = self._conanfile

        # Downstream consumer always higher priority
        generator_conf = conanfile.conf.get("tools.cmake.cmaketoolchain:generator")
        if generator_conf:
            return generator_conf

        # second priority: the recipe one:
        if recipe_generator:
            return recipe_generator

        # if not defined, deduce automatically the default one
        compiler = conanfile.settings.get_safe("compiler")
        compiler_version = conanfile.settings.get_safe("compiler.version")

        cmake_years = {'8': '8 2005',
                       '9': '9 2008',
                       '10': '10 2010',
                       '11': '11 2012',
                       '12': '12 2013',
                       '14': '14 2015',
                       '15': '15 2017',
                       '16': '16 2019',
                       '17': '17 2022',
                       '18': '18 2026'}

        if compiler == "msvc":
            if compiler_version is None:
                raise ConanException("compiler.version must be defined")
            vs_version = vs_ide_version(self._conanfile)
            return "Visual Studio %s" % cmake_years[vs_version]

        if use_win_mingw(conanfile):
            return "MinGW Makefiles"

        return "Unix Makefiles"



from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


class TestConditionalReqsTest:

    def test_conditional_requirements(self):
        conanfile = GenConanfile("hello", "0.1").with_settings("os", "build_type", "product")

        test_conanfile = '''
from conan import ConanFile

class TestConanLib(ConanFile):
    settings = "os", "build_type", "product"
    def requirements(self):
        self.output.info("TestSettings: %s, %s, %s"
                         % (self.settings.os, self.settings.build_type, self.settings.product))
        self.requires(self.tested_reference_str)

    def test(self):
        pass
'''
        client = TestClient()
        save(client.paths.settings_path_user, "product: [onion, potato]")
        client.save({"conanfile.py": conanfile,
                     "test_package/conanfile.py": test_conanfile})
        client.run("create . -s os=Windows -s product=onion -s build_type=Release")
        assert"hello/0.1 (test package): TestSettings: Windows, Release, onion", client.out



import platform
import textwrap

import pytest

from conan.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Windows", reason="Only windows")
class TestConanSettingsPreprocessor:

    def test_runtime_auto(self):
        c = TestClient()
        # Ensure that compiler.runtime is now in the default Win MSVC profile
        default_profile = c.load_home("profiles/default")
        assert "compiler.runtime" in default_profile

        conanfile = textwrap.dedent("""\
            from conan import ConanFile

            class HelloConan(ConanFile):
                name = "hello0"
                version = "0.1"
                settings = "os", "compiler", "build_type"

                def configure(self):
                    self.output.info(f"Runtime_type: {self.settings.compiler.runtime_type}")
            """)

        c.save({"conanfile.py": conanfile})
        c.run("create .")
        assert "Runtime_type: Release" in c.out
        c.run("create . -s build_type=Debug")
        assert "Runtime_type: Debug" in c.out

    def test_runtime_not_present_ok(self):
        c = TestClient()
        c.save({"conanfile.txt": ""})
        c.run("install .")
        default_settings = c.load_home("settings.yml")
        default_settings = default_settings.replace("runtime:", "# runtime:")
        default_settings = default_settings.replace("runtime_type:", "# runtime_type:")
        profile = "[settings]\nos=Windows\ncompiler=msvc\ncompiler.version=191"
        c.save_home({"settings.yml": default_settings,
                     "profiles/default": profile})
        # Ensure the runtime setting is not there anymore
        c.run('install . -s compiler.runtime="dynamic"', assert_error=True)
        assert "'settings.compiler.runtime' doesn't exist for 'msvc'" in c.out

        # Now install, the preprocessor shouldn't fail nor do anything
        c.run("install .")
        assert "Installing packages" in c.out



import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


def test_settings_user():
    c = TestClient()
    settings_user = textwrap.dedent("""\
        os:
            Windows:
                subsystem: [new_sub]
            Linux:
                new_versions: ["a", "b", "c"]
            new_os:
        new_global: ["42", "21"]
        """)
    save(os.path.join(c.cache_folder, "settings_user.yml"), settings_user)
    c.save({"conanfile.py": GenConanfile().with_settings("os").with_settings("new_global")})
    # New settings are there
    c.run("install . -s os=Windows -s os.subsystem=new_sub -s new_global=42")
    assert "new_global=42" in c.out
    assert "os.subsystem=new_sub" in c.out
    # Existing values of subsystem are still there
    c.run("install . -s os=Windows -s os.subsystem=msys2 -s new_global=42")
    assert "new_global=42" in c.out
    assert "os.subsystem=msys2" in c.out
    # Completely new values, not appended, but new, are there
    c.run("install . -s os=Linux -s os.new_versions=a -s new_global=42")
    assert "new_global=42" in c.out
    assert "os.new_versions=a" in c.out
    # Existing values of OSs are also there
    c.run("install . -s os=Macos -s new_global=42")
    assert "os=Macos" in c.out
    assert "new_global=42" in c.out


def test_settings_user_subdict():
    c = TestClient()
    settings_user = textwrap.dedent("""\
        other_new:
            other1:
            other2:
                version: [1, 2, 3]
        """)
    save(os.path.join(c.cache_folder, "settings_user.yml"), settings_user)
    c.save({"conanfile.py": GenConanfile().with_settings("other_new")})
    c.run("install . -s other_new=other1")
    assert "other_new=other1" in c.out
    c.run("install . -s other_new=other2 -s other_new.version=2")
    assert "other_new=other2" in c.out
    assert "other_new.version=2" in c.out


def test_settings_user_convert_list_dict():
    c = TestClient()
    settings_user = textwrap.dedent("""\
        arch:
            x86:
                subarch32:
                    a:
                        version: ["a1", "a2"]
                    b:
                        variant: ["b1", "b2"]
            x86_64:
                subarch: [1, 2, 3]
          """)
    save(os.path.join(c.cache_folder, "settings_user.yml"), settings_user)
    c.save({"conanfile.py": GenConanfile().with_settings("arch")})
    # check others are maintained
    c.run("install . -s arch=armv8")
    assert "arch=armv8" in c.out

    c.run("install . -s arch=x86 -s arch.subarch32=a -s arch.subarch32.version=a1")
    assert "arch=x86" in c.out
    assert "arch.subarch32=a" in c.out
    assert "arch.subarch32.version=a1" in c.out

    c.run("install . -s arch=x86_64 -s arch.subarch=2 ")
    assert "arch=x86_64" in c.out
    assert "arch.subarch=2" in c.out


def test_settings_user_error():
    c = TestClient()
    settings_user = textwrap.dedent("""\
        os:
            Windows:
                libc: null
        """)
    save(os.path.join(c.cache_folder, "settings_user.yml"), settings_user)
    c.run("profile show", assert_error=True)
    assert "ERROR: Definition of settings.yml 'settings.os.libc' cannot be null" in c.out


def test_settings_user_breaking_universal_binaries():
    # If you had a settings_user.yml with a custom architecture wit will error
    # in the Apple block of CMakeToolchain
    # https://github.com/conan-io/conan/issues/16086#issuecomment-2059118224
    c = TestClient()
    settings_user = textwrap.dedent("""\
        arch: [universal]
        """)
    save(os.path.join(c.cache_folder, "settings_user.yml"), settings_user)
    c.save({"conanfile.py": GenConanfile().with_settings("os").with_settings("arch").with_generator("CMakeToolchain")})
    c.run('install . -s="arch=universal"')
    assert "CMakeToolchain generated: conan_toolchain.cmake" in c.out



import os
import textwrap

from requests import Response

from conan.test.utils.tools import TestClient, TestRequester
from conan.test.utils.env import environment_update


class TestProxiesConfTest:

    def test_requester_with_host_specific_proxies(self):
        class MyHttpRequester(TestRequester):

            def get(self, _, **kwargs):
                resp = Response()
                # resp._content = b'{"results": []}'
                resp.status_code = 200
                resp._content = b''
                print(kwargs["proxies"])
                return resp

        client = TestClient(requester_class=MyHttpRequester)
        client.save_home({"global.conf": 'core.net.http:proxies = {"myproxykey": "myvalue"}'})
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import download

            class Pkg(ConanFile):
                settings = "os", "compiler"

                def source(self):
                    download(self, "MyUrl", "filename.txt")
        """)
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=foo --version=1.0")
        assert "{'myproxykey': 'myvalue'}" in client.out

    def test_new_proxy_exclude(self):

        class MyHttpRequester(TestRequester):

            def get(self, _, **kwargs):
                resp = Response()
                # resp._content = b'{"results": []}'
                resp.status_code = 200
                resp._content = b''
                print("is excluded!" if "proxies" not in kwargs else "is not excluded!")
                return resp

        client = TestClient(requester_class=MyHttpRequester)
        client.save_home({"global.conf":
             'core.net.http:no_proxy_match = ["MyExcludedUrl*", "*otherexcluded_one*"]\n'
             'core.net.http:proxies = {"http": "value"}'})
        for url in ("**otherexcluded_one***", "MyUrl", "MyExcludedUrl***", "**MyExcludedUrl***"):
            conanfile = textwrap.dedent("""
                from conan import ConanFile
                from conan.tools.files import download

                class Pkg(ConanFile):
                  settings = "os", "compiler"

                  def source(self):
                      download(self, "{}", "filename.txt")
                """).format(url)
            client.save({"conanfile.py": conanfile})
            client.run("create . --name=foo --version=1.0")
            if url in ("MyUrl", "**MyExcludedUrl***"):
                assert "is not excluded!" in client.out
            else:
                assert "is excluded!" in client.out

    def test_environ_kept(self):

        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import download

            class Pkg(ConanFile):
                settings = "os", "compiler"

                def source(self):
                    download(self, "http://foo.bar/file", "filename.txt")
            """)

        class MyHttpRequester(TestRequester):

            def get(self, _, **kwargs):
                resp = Response()
                # resp._content = b'{"results": []}'
                resp.status_code = 200
                resp._content = b''
                assert "HTTP_PROXY" in os.environ
                print("My requester!")
                return resp

        client = TestClient(requester_class=MyHttpRequester)
        client.save({"conanfile.py": conanfile})

        with environment_update({"HTTP_PROXY": "my_system_proxy"}):
            client.run("create . --name=foo --version=1.0")

        assert "My requester!" in client.out

    def test_environ_removed(self):
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import download

            class Pkg(ConanFile):
                settings = "os", "compiler"

                def source(self):
                    download(self, "http://MyExcludedUrl/file", "filename.txt")
            """)

        class MyHttpRequester(TestRequester):

            def get(self, _, **kwargs):
                resp = Response()
                # resp._content = b'{"results": []}'
                resp.status_code = 200
                resp._content = b''
                assert "HTTP_PROXY" not in os.environ
                assert "http_proxy" not in os.environ
                print("My requester!")
                return resp

        client = TestClient(requester_class=MyHttpRequester)
        client.save_home({"global.conf": 'core.net.http:clean_system_proxy = True'})

        with environment_update({"http_proxy": "my_system_proxy"}):
            client.save({"conanfile.py": conanfile})
            client.run("create . --name=foo --version=1.0")
            assert "My requester!" in client.out



import os
import sys


def list_folder_subdirs(basedir, level):
    ret = []
    for root, dirs, _ in os.walk(basedir):
        rel_path = os.path.relpath(root, basedir)
        if rel_path == ".":
            continue
        dir_split = rel_path.split(os.sep)
        if len(dir_split) == level:
            ret.append("/".join(dir_split))
            dirs[:] = []  # Stop iterate subdirs
    return ret


def path_exists(path, basedir):
    """Case sensitive, for windows, optional
    basedir for skip caps check for tmp folders in testing for example (returned always
    in lowercase for some strange reason)"""
    exists = os.path.exists(path)
    if not exists or sys.platform == "linux2":
        return exists

    path = os.path.normpath(path)
    path = os.path.relpath(path, basedir)
    chunks = path.split(os.sep)
    tmp = basedir

    for chunk in chunks:
        if chunk and chunk not in os.listdir(tmp):
            return False
        tmp = os.path.normpath(tmp + os.sep + chunk)
    return True


def relative_dirs(path):
    """ Walks a dir and return a list with the relative paths """
    ret = []
    for dirpath, _, fnames in os.walk(path):
        for filename in fnames:
            tmp = os.path.join(dirpath, filename)
            tmp = tmp[len(path) + 1:]
            ret.append(tmp)
    return ret



import platform
import pytest
import textwrap

from conan.test.utils.tools import TestClient

@pytest.mark.skipif(platform.system() not in  ["Darwin", "Linux"], reason="Autotools on Linux or macOS")
def test_crossbuild_triplet_from_conf():

    settings_yml = textwrap.dedent("""
        os:
            Linux:
            Windows:
        arch: [x86_64, hexagon]
        compiler:
            gcc:
                version: ["10", "11"]
                libcxx: [libstdc++11]
        build_type: [None, Debug, Release]
        """)

    host_profile = textwrap.dedent("""
        [settings]
        os=Linux
        arch=hexagon
        compiler=gcc
        compiler.version=10
        compiler.libcxx=libstdc++11
        build_type=Release
        [conf]
        tools.gnu:host_triplet=hexagon-acme-linux-gnu
    """)

    build_profile = textwrap.dedent("""
        [settings]
        os=Linux
        arch=x86_64
        compiler=gcc
        compiler.version=11
        compiler.libcxx=libstdc++11
        build_type=Release
    """)

    client = TestClient(path_with_spaces=False)
    client.save({client.paths.settings_path: settings_yml})
    client.save({"host_profile": host_profile})
    client.save({"build_profile": build_profile})

    client.run("new autotools_lib -d name=hello -d version=0.1")
    client.run("create . --profile:build=build_profile --profile:host=host_profile -tf=\"\"")

    assert "--host=hexagon-acme-linux-gnu" in client.out
    assert "checking host system type... hexagon-acme-linux-gnu" in client.out



import os
import platform
import re
import sys
import textwrap

import pytest

from conan.test.assets.sources import gen_function_cpp, gen_function_h
from conan.test.utils.tools import TestClient
from conan.api.model import RecipeReference
from test.functional.toolchains.meson._base import check_binary


@pytest.mark.tool("ninja")
@pytest.mark.tool("meson")
class TestMesonToolchain:

    def test_definition_of_global_options(self):
        t = TestClient()
        conanfile_py = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.meson import Meson, MesonToolchain


        class App(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            options = {{"shared": [True, False], "fPIC": [True, False], "msg": ["ANY"]}}
            default_options = {{"shared": False, "fPIC": True, "msg": "Hi everyone!"}}

            def config_options(self):
                if self.settings.os == "Windows":
                    self.options.rm_safe("fPIC")

            def configure(self):
                if self.options.shared:
                    self.options.rm_safe("fPIC")

            def layout(self):
                self.folders.generators = 'build/gen_folder'
                self.folders.build = "build"

            def generate(self):
                tc = MesonToolchain(self)
                tc.project_options["STRING_DEFINITION"] = "Text"
                tc.project_options["TRUE_DEFINITION"] = True
                tc.project_options["FALSE_DEFINITION"] = False
                tc.project_options["INT_DEFINITION"] = 42
                tc.project_options["ARRAY_DEFINITION"] = ["Text1", "Text2"]
                # Meson converts True/False into true/false boolean values
                tc.project_options["DYNAMIC"] = self.options.shared
                # This one will fail because HELLO_MSG = Hi everyone is an invalid value
                # The good one should be HELLO_MSG = "Hi everyone"
                tc.project_options["HELLO_MSG"] = {msg}
                tc.generate()

            def build(self):
                meson = Meson(self)
                meson.configure()
                meson.build(target='hello')
                meson.build(target='demo')
        """)

        meson_options_txt = textwrap.dedent("""
        option('STRING_DEFINITION', type : 'string', description : 'a string option')
        option('INT_DEFINITION', type : 'integer', description : 'an integer option', value: 0)
        option('FALSE_DEFINITION', type : 'boolean', description : 'a boolean option (false)')
        option('TRUE_DEFINITION', type : 'boolean', description : 'a boolean option (true)')
        option('ARRAY_DEFINITION', type : 'array', description : 'an array option')
        option('HELLO_MSG', type : 'string', description : 'message to print')
        option('DYNAMIC', type : 'boolean', description : 'shared to true or false')
        """)

        meson_build = textwrap.dedent("""
        project('tutorial', 'cpp')
        add_global_arguments('-DSTRING_DEFINITION="' + get_option('STRING_DEFINITION') + '"',
                             language : 'cpp')
        add_global_arguments('-DHELLO_MSG="' + get_option('HELLO_MSG') + '"', language : 'cpp')
        hello = library('hello', 'hello.cpp')
        executable('demo', 'main.cpp', link_with: hello)
        """)

        hello_h = gen_function_h(name="hello")
        hello_cpp = gen_function_cpp(name="hello", preprocessor=["STRING_DEFINITION"])
        app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])

        # Issue related: https://github.com/conan-io/conan/issues/14453
        # At first, let's add a raw option to see that it fails
        t.save({"conanfile.py": conanfile_py.format(msg="self.options.msg"),
                "meson.build": meson_build,
                "meson_options.txt": meson_options_txt,
                "hello.h": hello_h,
                "hello.cpp": hello_cpp,
                "main.cpp": app})
        t.run("build .", assert_error=True)
        # Using directly the options, the result is -> HELLO_MSG = Hi everyone, which is incorrect
        assert "WARN: deprecated: Please, do not use a Conan option value directly. " \
               "Convert 'options.shared' into" in t.out
        assert "WARN: deprecated: Please, do not use a Conan option value directly. " \
               "Convert 'options.msg' into" in t.out
        assert "Malformed value" in t.out
        # Let's transform the Conan option into other allowed data type to solve the issue
        t.save({"conanfile.py": conanfile_py.format(msg="str(self.options.msg)")})
        t.run("build .")
        content = t.load(os.path.join("build", "gen_folder", "conan_meson_native.ini"))
        assert "[project options]" in content
        assert "STRING_DEFINITION = 'Text'" in content
        assert "TRUE_DEFINITION = true" in content
        assert "FALSE_DEFINITION = false" in content
        assert "DYNAMIC = False" in content  # Meson transforms correctly this value into bool
        assert "HELLO_MSG = 'Hi everyone!'" in content
        assert "INT_DEFINITION = 42" in content
        assert "ARRAY_DEFINITION = ['Text1', 'Text2']" in content
        assert "[built-in options]" in content
        assert "buildtype = 'release'" in content

        t.run_command(os.path.join("build", "demo"))
        assert "hello: Release!" in t.out
        assert "STRING_DEFINITION: Text" in t.out
        assert "[properties]" in content
        assert "needs_exe_wrapper" not in content

        check_binary(t)

    def test_meson_default_dirs(self):
        t = TestClient()
        t.run("new meson_exe -d name=hello -d version=1.0")
        # t.run("new meson_exe -d name=hello -d version=1.0 -m meson_exe")

        meson_build = textwrap.dedent("""
        project('tutorial', 'cpp')
        # Creating specific library
        hello = library('hello', 'src/hello.cpp', install: true)
        # Creating specific executable
        executable('demo', 'src/main.cpp', link_with: hello, install: true)
        # Creating specific data in src/ (they're going to be exported)
        install_data(['src/file1.txt', 'src/file2.txt'])
        """)
        conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.meson import Meson
        class HelloConan(ConanFile):
            name = "hello"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            exports_sources = "meson.build", "src/*"
            generators = "MesonToolchain"
            package_type = "application"

            def layout(self):
                self.folders.build = "build"
                # Only adding "res" to resdirs
                self.cpp.package.resdirs = ["res"]

            def build(self):
                meson = Meson(self)
                meson.configure()
                meson.build()

            def package(self):
                meson = Meson(self)
                meson.install()
        """)
        test_conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.build import cross_building
        class HelloTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"

            def requirements(self):
                self.requires(self.tested_reference_str)

            def test(self):
                if not cross_building(self):
                    self.run("demo", env="conanrun")
        """)
        # Replace meson.build, conanfile.py and test_package/conanfile.py
        t.save({"meson.build": meson_build,
                "conanfile.py": conanfile,
                "test_package/conanfile.py": test_conanfile,
                "src/file1.txt": "", "src/file2.txt": ""})
        t.run("create . -c tools.build:verbosity=quiet -c tools.compilation:verbosity=verbose")
        # Check verbosity control
        assert "unrecognized arguments" not in t.out
        assert re.search("meson compile .*? --verbose", t.out)
        assert re.search("meson install .*? --quiet", t.out)

        # Check if all the files are in the final directories
        ref = RecipeReference.loads("hello/1.0")
        pref = t.get_latest_package_reference(ref)
        package_folder = t.get_latest_pkg_layout(pref).package()
        if platform.system() == "Windows":
            assert os.path.exists(os.path.join(package_folder, "lib", "hello.lib"))
            assert os.path.exists(os.path.join(package_folder, "bin", "hello.dll"))
            assert os.path.exists(os.path.join(package_folder, "bin", "demo.exe"))
        else:
            ext = "dylib" if platform.system() == "Darwin" else "so"
            assert os.path.exists(os.path.join(package_folder, "bin", "demo"))
            assert os.path.exists(os.path.join(package_folder, "lib", "libhello." + ext))
        # res/tutorial -> tutorial is being added automatically by Meson
        assert os.path.exists(os.path.join(package_folder, "res", "tutorial", "file1.txt"))
        assert os.path.exists(os.path.join(package_folder, "res", "tutorial", "file2.txt"))


@pytest.mark.tool("meson")
@pytest.mark.skipif(sys.version_info.minor < 8, reason="Latest Meson versions needs Python >= 3.8")
def test_meson_and_additional_machine_files_composition():
    """
    Testing when users wants to append their own meson machine files and override/complement some
    sections from Conan file ones.

    See more information in Meson web page: https://mesonbuild.com/Machine-files.html

    In this test, we're overriding only the Meson section ``[binaries]`` for instance.
    """
    profile = textwrap.dedent("""
        [settings]
        os=Windows
        arch=x86_64
        compiler=gcc
        compiler.version=9
        compiler.cppstd=17
        compiler.libcxx=libstdc++11
        build_type=Release

        [conf]
        tools.meson.mesontoolchain:extra_machine_files=["myfilename.ini"]
   """)
    myfilename = textwrap.dedent("""
    [project options]
    my_option = 'fake-option'
    """)
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import Meson
    class Pkg(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        generators = "MesonToolchain"
        def build(self):
            meson = Meson(self)
            meson.configure()
            meson.build()
    """)
    client = TestClient()
    client.save({"conanfile.py": conanfile,
                 "build/myfilename.ini": myfilename,
                 "meson.build": "project('tutorial', 'cpp')",  # dummy one
                 "profile": profile})

    client.run("install . -pr:h=profile -pr:b=profile")
    client.run("build . -pr:h=profile -pr:b=profile", assert_error=True)
    # Checking the order of the appended user file (the order matters)
    match = re.search(r"meson setup --native-file .* --native-file \"myfilename\.ini\"", client.out)
    assert match


@pytest.mark.tool("ninja")
@pytest.mark.tool("meson")
@pytest.mark.skipif(platform.system() != "Windows", reason="Only for Windows")
@pytest.mark.skipif(sys.version_info.minor < 8, reason="Latest Meson versions needs Python >= 3.8")
def test_meson_using_prefix_path_in_application():
    """
    Issue related https://github.com/conan-io/conan/issues/14213
    """
    meson_build = textwrap.dedent("""
    project('myhello ', 'c')
    executable('myhello', 'src/main.c', install: true)

    prefix_dir = get_option('prefix')
    cfg_var = configuration_data()
    cfg_var.set_quoted('MYHELLO_PREFIX', prefix_dir)

    config_file = configure_file(
        configuration: cfg_var,
        output: 'config.h'
    )
    """)
    main_c = textwrap.dedent("""
    #include <config.h>

    int main(void) {
        return 0;
    }

    char *issue_func() {
        return (MYHELLO_PREFIX);
    }
    """)
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import Meson
    from conan.tools.layout import basic_layout

    class myhelloConan(ConanFile):
        name = "demo"
        version = "0.1"
        settings = "os", "compiler", "build_type", "arch"
        exports_sources = "meson.build", "*.c"
        package_type = "application"
        generators = "MesonToolchain"

        def layout(self):
            basic_layout(self)

        def build(self):
            meson = Meson(self)
            meson.configure()
            meson.build()
    """)
    client = TestClient()
    client.save({"conanfile.py": conanfile,
                 "src/main.c": main_c,
                 "meson.build": meson_build})
    client.run("build .")
    assert "unrecognized character escape sequence" not in str(client.out)  # if Visual
    assert "unknown escape sequence" not in str(client.out)  # if mingw
